from onelya_sdk.utils import get_datetime, get_array, get_item, get_money
from onelya_sdk.wrapper.types import ProviderPaymentForm
from .requests import (OrderFullCustomerRequest, OrderCustomerDocuments, AeroexpressReservationRequest,
                       AeroexpressAutoReturnRequest)
from onelya_sdk.wrapper import (OrderCreateReservationCustomerResponse, OrderCustomerResponse, ReservationResponse,
                                AeroexpressConfirmResponse, AeroexpressAutoReturnResponse)

CREATE_METHOD = 'Order/V1/Reservation/Create'
CONFIRM_METHOD = 'Order/V1/Reservation/Confirm'
BLANK_METHOD = 'Order/V1/Reservation/Blank'
CANCEL_METHOD = 'Order/V1/Reservation/Cancel'
VOID_METHOD = 'Order/V1/Reservation/Void'
AUTO_RETURN_METHOD = 'Order/V1/Reservation/AutoReturn'


class Reservation(object):

    def __init__(self, request_wrapper):
        self.request_wrapper = request_wrapper

    def create(self, customers: OrderFullCustomerRequest, reservation_items: AeroexpressReservationRequest,
               contact_phone=None, contact_emails=None):
        response = self.request_wrapper.make_request(CREATE_METHOD, customers=customers, contact_phone=contact_phone,
                                                     reservation_items=reservation_items, contact_emails=contact_emails)
        return CreateReservation(response)

    def confirm(self, order_id: int, provider_payment_form: ProviderPaymentForm,
                order_customer_ids: 'list of int'=None,
                order_customer_documents: 'list of OrderCustomerDocuments'=None):
        response = self.request_wrapper.make_request(CONFIRM_METHOD, order_id=order_id, order_customer_ids=order_customer_ids,
                                                     order_customer_documents=order_customer_documents,
                                                     provider_payment_form=provider_payment_form)
        return Confirm(response)

    def blank(self, order_id: int, order_item_id: int=None, retrieve_main_services: bool=True, retrieve_upsales: bool=True):

        response = self.request_wrapper.make_request(BLANK_METHOD, order_id=order_id, order_item_id=order_item_id,
                                                     retrieve_main_services=retrieve_main_services,
                                                     retrieve_upsales=retrieve_upsales)
        return Blank(response)

    def cancel(self, order_id: int, order_item_ids: 'list of int'=None, order_customer_ids: 'list of int'=None):
        self.request_wrapper.make_request(CANCEL_METHOD, order_id=order_id, order_item_ids=order_item_ids,
                                          order_customer_ids=order_customer_ids)
        return True

    def void(self, order_id: int, order_item_ids: 'list of int'=None, order_customer_ids: 'list of int'=None):
        response = self.request_wrapper.make_request(VOID_METHOD, order_id=order_id, order_item_ids=order_item_ids,
                                                     order_customer_ids=order_customer_ids)
        return Void(response)

    def auto_return(self, order_item_id: int, agent_reference_id: str=None):
        response = self.request_wrapper.make_request(AUTO_RETURN_METHOD,
                                                     service_auto_return_request=AeroexpressAutoReturnRequest(
                                                         order_item_id, agent_reference_id))
        return AutoReturn(response)


class CreateReservation(object):
    def __init__(self, json_data):
        self.order_id = json_data.get('OrderId')
        self.amount = get_money(json_data.get('Amount'))
        self.confirm_till = get_datetime(json_data.get('ConfirmTill'))
        self.customers = get_array(json_data.get('Customers'), OrderCreateReservationCustomerResponse)
        self.reservation_results = get_array(json_data.get('ReservationResults'), ReservationResponse)

        self.json_data = json_data


class ProlongReservation(object):
    def __init__(self, json_data):
        self.order_id = json_data.get('OrderId')
        self.confirm_till = get_datetime(json_data.get('ConfirmTill'))

        self.json_data = json_data


class Confirm(object):
    def __init__(self, json_data):
        self.order_id = json_data.get('OrderId')
        self.customers = get_array(json_data.get('Customers'),  OrderCustomerResponse)
        self.confirm_results = get_array(json_data.get('ConfirmResults'), AeroexpressConfirmResponse)

        self.json_data = json_data


class Blank(object):
    def __init__(self, data):
        self.__data = data

    def save_blank(self, path):
        open(path, 'wb').write(self.content)

    @property
    def content(self):
        return self.__data.content


class Void(object):
    def __init__(self, json_data):
        self.order_id = json_data.get('OrderId')

        self.json_data = json_data


class AutoReturn(object):
    def __init__(self, json_data):
        self.service_return_response = get_item(json_data.get('ServiceReturnResponse'), AeroexpressAutoReturnResponse)

        self.json_data = json_data



from .requests import ServiceReturnAmountRequest
from onelya_sdk.utils import get_datetime, get_array, get_item, get_bool_item, get_money
from onelya_sdk.wrapper.types import ProlongReservationType, ProviderPaymentForm, ReturnTarget
from .requests import (OrderFullCustomerRequest, RailwayReservationRequest, OrderCustomerDocuments,
                       ServiceAutoReturnRequest, ServiceAddUpsaleRequest)
from onelya_sdk.wrapper import (OrderCreateReservationCustomerResponse, ReservationResponse, OrderCustomerResponse,
                                RailwayConfirmResponse, RailwayReturnAmountResponse, RailwayAutoReturnResponse,
                                CustomerUpsaleOperationResult, RailwayBlankInfo)

CREATE_METHOD = 'Order/V1/Reservation/Create'
PROLONG_RESERVATION_METHOD = 'Order/V1/Reservation/ProlongReservation'
CONFIRM_METHOD = 'Order/V1/Reservation/Confirm'
BLANK_METHOD = 'Order/V1/Reservation/Blank'
CANCEL_METHOD = 'Order/V1/Reservation/Cancel'
RETURN_AMOUNT_METHOD = 'Order/V1/Reservation/ReturnAmount'
AUTO_RETURN_METHOD = 'Order/V1/Reservation/AutoReturn'
ADD_UPSALE_METHOD = 'Order/V1/Reservation/AddUpsale'
REFUSE_UPSALE_METHOD = 'Order/V1/Reservation/RefuseUpsale'

UPDATE_BLANKS_METHOD = 'Railway/V1/Reservation/UpdateBlanks'
ELECTRONIC_REGISTRATION_METHOD = 'Railway/V1/Reservation/ElectronicRegistration'
MEAL_OPTION_METHOD = 'Railway/V1/Reservation/MealOption'
BLANK_AS_HTML_METHOD = 'Railway/V1/Reservation/BlankAsHtml'


class Reservation(object):

    def __init__(self, request_wrapper):
        self.request_wrapper = request_wrapper

    def create(self, customers: OrderFullCustomerRequest, reservation_items: RailwayReservationRequest,
               contact_phone=None, contact_emails=None):
        response = self.request_wrapper.make_request(CREATE_METHOD, customers=customers, contact_phone=contact_phone,
                                                     reservation_items=reservation_items, contact_emails=contact_emails)
        return CreateReservation(response)

    def prolong_reservation(self, order_id: int, order_item_ids: 'list of int'=None,
                            prolong_reservation_type: ProlongReservationType=ProlongReservationType.RAILWAY_THREE_HOURS_RESERVATION):
        response = self.request_wrapper.make_request(PROLONG_RESERVATION_METHOD, order_id=order_id,
                                                     prolong_reservation_type=prolong_reservation_type,
                                                     order_item_ids=order_item_ids)
        return ProlongReservation(response)

    def confirm(self, order_id: int, provider_payment_form: ProviderPaymentForm,
                order_customer_ids: 'list of int'=None,
                order_customer_documents: 'lisf of OrderCustomerDocuments'=None):
        response = self.request_wrapper.make_request(CONFIRM_METHOD, order_id=order_id, order_customer_ids=order_customer_ids,
                                                     order_customer_documents=order_customer_documents,
                                                     provider_payment_form=provider_payment_form)
        return Confirm(response)

    def blank(self, order_id: int, order_item_id: int, order_item_ids: 'list of int', retrieve_main_services: bool=True,
              retrieve_upsales: bool=True):

        response = self.request_wrapper.make_request(BLANK_METHOD, order_id=order_id, order_item_id=order_item_id,
                                                     order_item_ids=order_item_ids,
                                                     retrieve_main_services=retrieve_main_services,
                                                     retrieve_upsales=retrieve_upsales)
        return Blank(response)

    def cancel(self, order_id: int, order_item_ids: 'list of int'=None, order_customer_ids: 'list of int'=None):
        self.request_wrapper.make_request(CANCEL_METHOD, order_id=order_id, order_item_ids=order_item_ids,
                                          order_customer_ids=order_customer_ids)
        return True

    def return_amount(self, check_document_number: str, order_item_id: int, order_item_blank_ids: 'list of int'=None,
                      return_target: ReturnTarget=None):
        response = self.request_wrapper.make_request(RETURN_AMOUNT_METHOD,
                                                     service_return_amount_request=ServiceReturnAmountRequest(
                                                         check_document_number, order_item_id, order_item_blank_ids,
                                                         return_target))
        return ReturnAmount(response)

    def auto_return(self, check_document_number: str, order_item_id: int, order_item_blank_ids: 'list of int'=None,
                    agent_reference_id: str=None):
        response = self.request_wrapper.make_request(AUTO_RETURN_METHOD,
                                                     service_auto_return_request=ServiceAutoReturnRequest(
                                                         check_document_number, order_item_id, order_item_blank_ids,
                                                         agent_reference_id))
        return AutoReturn(response)

    def add_upsale(self, order_id: int, order_item_id: int=None, service_add_upsale_request: ServiceAddUpsaleRequest=None):
        response = self.request_wrapper.make_request(ADD_UPSALE_METHOD, order_id=order_id, order_item_id=order_item_id,
                                                     service_add_upsale_request=service_add_upsale_request)
        return AddUpsale(response)

    def refuse_upsale(self, order_id: int=None, order_item_id: int=None, order_customer_ids: 'list of int'=None):
        response = self.request_wrapper.make_request(REFUSE_UPSALE_METHOD, order_id=order_id, order_item_id=order_item_id,
                                                     order_customer_ids=order_customer_ids)
        return RefuseUpsale(response)

    def update_blanks(self, order_item_id: int):
        response = self.request_wrapper.make_request(UPDATE_BLANKS_METHOD, order_item_id=order_item_id)
        return UpdateBlanks(response)

    def electronic_registration(self, order_item_id: int, set: bool, order_item_blank_ids: 'list of int'=None,
                                send_notification: bool=False):
        response = self.request_wrapper.make_request(ELECTRONIC_REGISTRATION_METHOD, order_item_id=order_item_id,
                                                     set=set, order_item_blank_ids=order_item_blank_ids,
                                                     send_notification=send_notification)
        return ElectronicRegistration(response)

    def meal_option(self, order_item_id: int, meal_option_code: str, order_item_blank_id: int):
        response = self.request_wrapper.make_request(MEAL_OPTION_METHOD, order_item_id=order_item_id,
                                                     meal_option_code=meal_option_code,
                                                     order_item_blank_id=order_item_blank_id)
        return MealOption(response)

    def blank_as_html(self, order_item_id: int, order_item_ids: 'list of int'=None):
        response = self.request_wrapper.make_request(BLANK_AS_HTML_METHOD, order_item_id=order_item_id,
                                                     order_item_ids=order_item_ids)
        return BlankAsHtml(response)


class CreateReservation(object):
    def __init__(self, json_data):
        self.order_id = get_item(json_data.get('OrderId'), int)
        self.amount = get_money(json_data.get('Amount'))
        self.contact_phone = json_data.get('ContactPhone')
        self.contact_emails = json_data.get('ContactEmails')
        self.confirm_till = get_datetime(json_data.get('ConfirmTill'))
        self.customers = get_array(json_data.get('Customers'), OrderCreateReservationCustomerResponse)
        self.reservation_results = get_array(json_data.get('ReservationResults'), ReservationResponse)

        self.json_data = json_data


class ProlongReservation(object):
    def __init__(self, json_data):
        self.order_id = get_item(json_data.get('OrderId'), int)
        self.confirm_till = get_datetime(json_data.get('ConfirmTill'))

        self.json_data = json_data


class Confirm(object):
    def __init__(self, json_data):
        self.order_id = get_item(json_data.get('OrderId'), int)
        self.customers = get_array(json_data.get('Customers'),  OrderCustomerResponse)
        self.confirm_results = get_array(json_data.get('ConfirmResults'), RailwayConfirmResponse)

        self.json_data = json_data


class Blank(object):
    def __init__(self, data):
        self.__data = data

    def save_blank(self, path):
        open(path, 'wb').write(self.content)

    @property
    def content(self):
        return self.__data.content


class ReturnAmount(object):
    def __init__(self, json_data):
        self.service_return_response = get_item(json_data.get('ServiceReturnResponse'), RailwayReturnAmountResponse)

        self.json_data = json_data


class AutoReturn(object):
    def __init__(self, json_data):
        self.service_return_response = get_item(json_data.get('ServiceReturnResponse'), RailwayAutoReturnResponse)

        self.json_data = json_data


class AddUpsale(object):
    def __init__(self, json_data):
        self.order_id = get_item(json_data.get('OrderId'), int)
        self.upsale_results = get_array(json_data.get('UpsaleResults'), CustomerUpsaleOperationResult)

        self.json_data = json_data


class RefuseUpsale(object):
    def __init__(self, json_data):
        self.order_id = get_item(json_data.get('OrderId'), int)

        self.json_data = json_data


class UpdateBlanks(object):
    def __init__(self, json_data):
        self.blanks = get_array(json_data.get('Blanks'), RailwayBlankInfo)
        self.is_modified = get_bool_item(json_data.get('IsModified'))

        self.json_data = json_data


class ElectronicRegistration(object):
    def __init__(self, json_data):
        self.expiration_electronic_registration_date_time = get_datetime(json_data.get('ExpirationElectronicRegistrationDateTime'))
        self.blanks = get_array(json_data.get('Blanks'), RailwayBlankInfo)

        self.json_data = json_data


class MealOption(object):
    def __init__(self, json_data):
        self.meal_option_code = json_data.get('MealOptionCode')

        self.json_data = json_data


class BlankAsHtml(object):
    def __init__(self, data):
        self.__data = data

    @property
    def html(self):
        return self.__data.text



from datetime import datetime
from onelya_sdk.utils import get_array, get_item, get_bool_item
from onelya_sdk.wrapper.types import CarGrouping, PricingTariffType, CarType, SpecialPlacesDemand
from onelya_sdk.wrapper import (FeeCalculation, TrainPriceInfo, StationClarifying, TrainInfo, Route,
                                CarPriceInfo, ScheduleInfo, TrainRouteRoute, TrainPricingResponse,
                                RouteReferenced, MealOption)

TRAIN_PRICING_METHOD = 'Railway/V1/Search/TrainPricing'
CAR_PRICING_METHOD = 'Railway/V1/Search/CarPricing'
SCHEDULE_METHOD = 'Railway/V1/Search/Schedule'
TRAIN_ROUTE_METHOD = 'Railway/V1/Search/TrainRoute'
ROUTES_METHOD = 'Railway/V1/Search/Routes'
ROUTE_RICING_METHOD = 'Railway/V1/Search/RoutePricing'
SEARCH_MEAL_METHOD = 'Railway/V1/Search/SearchMeal'


class Search(object):
    def __init__(self, request_wrapper):
        self.__request_wrapper = request_wrapper

    def train_pricing(self, origin: str, destination: str, departure_date: datetime, time_from: int=12,
                      time_to: int=24, car_grouping: CarGrouping=CarGrouping.GROUP,
                      special_places_demand: SpecialPlacesDemand=SpecialPlacesDemand.NO_VALUE):
        response = self.__request_wrapper.make_request(TRAIN_PRICING_METHOD, origin=origin, destination=destination,
                                                       car_grouping=car_grouping, time_from=time_from, time_to=time_to,
                                                       departure_date=departure_date,
                                                       special_places_demand=special_places_demand)
        return TrainPricing(response)

    def car_pricing(self, origin_code: str, destination_code: str, departure_date: datetime, train_number: str,
                    car_type: CarType=None, tariff_type: PricingTariffType=None,
                    special_places_demand: SpecialPlacesDemand = SpecialPlacesDemand.NO_VALUE):
        response = self.__request_wrapper.make_request(CAR_PRICING_METHOD, origin_code=origin_code,
                                                       destination_code=destination_code, departure_date=departure_date,
                                                       train_number=train_number, car_type=car_type,
                                                       tariff_type=tariff_type,
                                                       special_places_demand=special_places_demand)
        return CarPricing(response)

    def schedule(self, origin: str, destination: str, time_from: int=12, time_to: int=24, departure_date: datetime=None):
        response = self.__request_wrapper.make_request(SCHEDULE_METHOD, origin=origin, destination=destination,
                                                       departure_date=departure_date, time_from=time_from,
                                                       time_to=time_to)
        return Schedule(response)

    def train_route(self, train_number: str, origin: str, destination: str, departure_date: datetime):
        response = self.__request_wrapper.make_request(TRAIN_ROUTE_METHOD, train_number=train_number, origin=origin,
                                                       destination=destination, departure_date=departure_date)
        return TrainRoute(response)

    def routes(self, origin: str, destination: str, departure_date: datetime, min_change_time: int=60,
               max_change_time: int=360, first_change_only: bool=True):
        response = self.__request_wrapper.make_request(ROUTES_METHOD, origin=origin, first_change_only=first_change_only,
                                                       departure_date=departure_date, min_change_time=min_change_time,
                                                       max_change_time=max_change_time, destination=destination)
        return Routes(response)

    def route_pricing(self, origin_code: str, destination_code: str, departure_date: datetime):
        response = self.__request_wrapper.make_request(ROUTE_RICING_METHOD, destination_code=destination_code,
                                                       origin_code=origin_code, departure_date=departure_date)
        return RoutePricing(response)

    def search_meal(self, car_type:CarType, carrier_code: str, meal_group: str, country_code: str,
                    departure_date_time: datetime=None):
        response = self.__request_wrapper.make_request(SEARCH_MEAL_METHOD, car_type=car_type, meal_group=meal_group,
                                                       carrier_code=carrier_code, country_code=country_code,
                                                       departure_date_time=departure_date_time)
        return SearchMeal(response)


class TrainPricing(object):
    def __init__(self, json_data):
        self.origin_code = json_data.get('OriginCode')
        self.origin_station_code = json_data.get('OriginStationCode')
        self.origin_time_zone_difference = get_item(json_data.get('OriginTimeZoneDifference'), int)
        self.destination_code = json_data.get('DestinationCode')
        self.destination_station_code = json_data.get('DestinationStationCode')
        self.destination_time_zone_difference = get_item(json_data.get('DestinationTimeZoneDifference'), int)
        self.trains = get_array(json_data.get('Trains'), TrainPriceInfo)
        self.departure_time_description = json_data.get('DepartureTimeDescription')
        self.arrival_time_description = json_data.get('ArrivalTimeDescription')
        self.is_from_ukrain = get_bool_item(json_data.get('IsFromUkrain'))
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)
        self.not_all_trains_returned = get_bool_item(json_data.get('NotAllTrainsReturned'))
        self.station_clarifying = get_item(json_data.get('StationClarifying'), StationClarifying)
        self.booking_system = json_data.get('BookingSystem')
        self.id = get_item(json_data.get('Id'), int)
        self.route_policy = json_data.get('RoutePolicy')

        self.json_data = json_data


class CarPricing(object):
    def __init__(self, json_data):
        self.origin_code = json_data.get('OriginCode')
        self.origin_time_zone_difference = get_item(json_data.get('OriginTimeZoneDifference'), int)
        self.destination_code = json_data.get('DestinationCode')
        self.destination_time_zone_difference = get_item(json_data.get('DestinationTimeZoneDifference'), int)
        self.cars = get_array(json_data.get('Cars'), CarPriceInfo)
        self.route_policy = json_data.get('RoutePolicy')
        self.train_info = get_item(json_data.get('TrainInfo'), TrainInfo)
        self.is_from_ukrain = get_bool_item(json_data.get('IsFromUkrain'))
        self.allowed_document_types = json_data.get('AllowedDocumentTypes')
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)
        self.booking_system = json_data.get('BookingSystem')

        self.json_data = json_data


class Schedule(object):
    def __init__(self, json_data):
        self.origin_station_code = json_data.get('OriginStationCode')
        self.origin_time_zone_difference = get_item(json_data.get('OriginTimeZoneDifference'), int)
        self.destination_station_code = json_data.get('DestinationStationCode')
        self.destination_time_zone_difference = get_item(json_data.get('DestinationTimeZoneDifference'), int)
        self.route_policy = json_data.get('RoutePolicy')
        self.schedules = get_array(json_data.get('Schedules'), ScheduleInfo)
        self.station_clarifying = get_item(json_data.get('StationClarifying'), StationClarifying)
        self.not_all_trains_returned = get_bool_item(json_data.get('NotAllTrainsReturned'))

        self.json_data = json_data


class TrainRoute(object):
    def __init__(self, json_data):
        self.routes = get_array(json_data.get('Routes'), TrainRouteRoute)

        self.json_data = json_data


class Routes(object):
    def __init__(self, json_data):
        self.routes = get_array(json_data.get('Routes'), Route)

        self.json_data = json_data


class RoutePricing(object):
    def __init__(self, json_data):
        self.train_pricings = get_array(json_data.get('TrainPricings'), TrainPricingResponse)
        self.routes = get_array(json_data.get('Routes'), RouteReferenced)
        self.not_all_train_pricings_returned = json_data.get('NotAllTrainPricingsReturned')

        self.json_data = json_data


class SearchMeal(object):
    def __init__(self, json_data):
        self.meal_options = get_array(json_data.get('MealOptions'), MealOption)

        self.json_data = json_data



from datetime import datetime
from onelya_sdk.wrapper.types import MealTime


class AdditionalMeal(object):
    def __init__(self, meal_time: MealTime, meal_option_code: str):
        self.meal_time = meal_time
        self.meal_option_code = meal_option_code



from datetime import datetime


def get_money(item):
    if item not in ['', 'Unknown', 'NoValue', None]:
        return int(item * 100)
    return None

def get_array(items, item_class=str):
    if type(items) is list:
        return [item_class(item) for item in items]
    return []


def get_array_from_str(items):
    if items not in ['', 'Unknown', 'NoValue', None]:
        if ', ' in items:
            items = items.split(', ')
        else:
            items = items.split(',')
        if all([item.isdigit() for item in items]):
            return [int(item) for item in items]
        return [str(item) for item in items]


def get_item(item, item_class):
    if item is not None:
        return item_class(item)
    return None


def get_datetime(item):
    if item is not None:
        return datetime.strptime(item, '%Y-%m-%dT%X')
    return None


def get_datetime_array(items):
    if items is not None:
        return [datetime.strptime(item, '%Y-%m-%dT%X') for item in items]
    return None


def str_datetime(item):
    if item is not None:
        return item.strftime('%Y-%m-%dT%X')
    return None


def get_bool_item(item):
    if item is not None:
        return bool(item)
    return False



from datetime import datetime
from onelya_sdk.utils import get_array, get_item, get_money
from onelya_sdk.wrapper.types import ProviderPaymentForm
from onelya_sdk.wrapper import (AdditionalMealOption, RateValue, AdditionalMealOption)

PRICING_METHOD = 'Railway/V1/AdditionalMeal/Pricing'
PURCHASE_METHOD = 'Railway/V1/AdditionalMeal/Purchase'
RETURN_METHOD = 'Railway/V1/AdditionalMeal/Return'


class AdditionalMeal(object):
    def __init__(self, request_wrapper):
        self.__request_wrapper = request_wrapper

    def pricing(self, order_item_id: int):
        response = self.__request_wrapper.make_request(PRICING_METHOD, order_item_id=order_item_id)
        return Pricing(response)

    def purchase(self, order_item_id: int, order_item_blank_id: int, meal_times: 'list of MealTimes',
                 agent_payment_id: str, agent_reference_id: str, provider_payment_form: ProviderPaymentForm):
        response = self.__request_wrapper.make_request(PURCHASE_METHOD, order_item_id=order_item_id,
                                                       order_item_blank_id=order_item_blank_id,
                                                       meal_times=meal_times,
                                                       agent_payment_id=agent_payment_id,
                                                       agent_reference_id=agent_reference_id,
                                                       provider_payment_form=provider_payment_form)
        return Purchase(response)

    def return_(self, order_item_id: int, agent_reference_id: str):
        response = self.__request_wrapper.make_request(RETURN_METHOD, order_item_id=order_item_id,
                                                       agent_reference_id=agent_reference_id)
        return Return(response)


class Pricing(object):
    def __init__(self, json_data):
        self.meal_options = get_array(json_data.get('MealOptions'), AdditionalMealOption)

        self.json_data = json_data


class Purchase(object):
    def __init__(self, json_data):
        self.order_item_id = get_item(json_data.get('OrderItemId'), int)
        self.vat_rate_values = get_array(json_data.get('VatRateValues'), RateValue)
        self.meal_times = get_array(json_data.get('MealTimes'), AdditionalMealOption)
        self.agent_reference_id = json_data.get('AgentReferenceId')

        self.json_data = json_data


class Return(object):
    def __init__(self, json_data):
        self.order_item_id = get_item(json_data.get('OrderItemId'), int)
        self.amount = get_money(json_data.get('Amount'))
        self.agent_reference_id = json_data.get('AgentReferenceId')

        self.json_data = json_data



from datetime import datetime
from onelya_sdk.utils import get_array, get_datetime, get_money
from onelya_sdk.wrapper.types import OperationType, ProviderPaymentForm
from onelya_sdk.wrapper import OrderCustomerInfo, AeroexpressFullOrderItemInfo, AeroexpressShortOrderInfo

ORDER_INFO_METHOD = 'Order/V1/Info/OrderInfo'
ORDER_LIST_METHOD = 'Order/V1/Info/OrderList'


class Info(object):
    def __init__(self, request_wrapper):
        self.__request_wrapper = request_wrapper

    def info(self, order_id: int=None, agent_reference_id: str=None):
        response = self.__request_wrapper.make_request(ORDER_INFO_METHOD, order_id=order_id,
                                                       agent_reference_id=agent_reference_id)
        return OrderInfo(response)

    def list(self, date: datetime=None, operation_type: OperationType=None,
             provider_payment_form: ProviderPaymentForm=None, is_externally_loaded: bool=None):
        response = self.__request_wrapper.make_request(ORDER_LIST_METHOD, date=date, operation_type=operation_type,
                                                       provider_payment_form=provider_payment_form,
                                                       is_externally_loaded=is_externally_loaded)
        return OrderList(response)


class OrderInfo(object):
    def __init__(self, json_data):
        self.order_customers = get_array(json_data.get('OrderCustomers'), OrderCustomerInfo)
        self.order_items = get_array(json_data.get('OrderItems'), AeroexpressFullOrderItemInfo)

        self.order_id = json_data.get('OrderId')
        self.amount = get_money(json_data.get('Amount'))
        self.contact_phone = json_data.get('ContactPhone')
        self.contact_emails = json_data.get('ContactEmails')
        self.created = get_datetime(json_data.get('Created'))
        self.confirmed = get_datetime(json_data.get('Confirmed'))
        self.pos_sys_name = json_data.get('PosSysName')

        self.json_data = json_data


class OrderList(object):
    def __init__(self, json_data):
        self.orders = get_array(json_data.get('Orders'), AeroexpressShortOrderInfo)

        self.json_data = json_data



from .reservation.requests import (OrderFullCustomerRequest, RailwayReservationRequest,
                                   RailwayPassengerRequest, ServiceAddUpsaleRequest, ProductRequest)
from .additional_meal.requests import AdditionalMeal





class OnelyaAPIError(Exception):
    __slots__ = ['code', 'message', 'message_params', 'request_data']

    def __init__(self, method, error_data, request_data):
        super(OnelyaAPIError, self).__init__()
        self.method = method.replace("/", "-")
        self.error_data = error_data
        self.request_data = request_data

        self.code = error_data.get('Code')
        self.message = error_data.get('Message').replace('request.', '')
        try:
            self.message_params = self.__prettify_message_params(error_data, request_data)
        except:
            self.message_params = 'Can\'t prettify message'

    @staticmethod
    def __prettify_message_params(error_data, request_data):
        message_params = error_data.get('MessageParams', None)
        if message_params is not None:
            json_message_params = {}
            for message in message_params:
                for item in message.split(', '):
                    param_key = item.split('.')[1].split(' =')[0]
                    json_message_params[param_key] = request_data[param_key]
            return json_message_params
        return message_params

    def __str__(self):
        error_message = '''Code: {self.code}
        Message: {self.message}
        MessageParams: {self.message_params}
        Docs: https://test.onelya.ru/ApiDocs/Api?apiId={self.method}'''.format(self=self)
        return error_message



from datetime import datetime
from onelya_sdk.utils import get_array
from onelya_sdk.wrapper.types import TransportNodeType
from onelya_sdk.wrapper import TransportNode, City, Country, Region

TRANSPORT_NODES_METHOD = 'Info/V1/References/TransportNodes'
CITIES_METHOD = 'Info/V1/References/Cities'
COUNTRIES_METHOD = 'Info/V1/References/Countries'
REGIONS_METHOD = 'Info/V1/References/Regions'


class References(object):
    def __init__(self, request_wrapper):
        self.__request_wrapper = request_wrapper

    def transport_nodes(self, last_updated: datetime=None, type: TransportNodeType=None):
        response = self.__request_wrapper.make_request(TRANSPORT_NODES_METHOD, last_updated=last_updated, type=type)
        return TransportNodes(response)

    def cities(self, last_updated: datetime=None):
        response = self.__request_wrapper.make_request(CITIES_METHOD, last_updated=last_updated)
        return Cities(response)

    def countries(self, last_updated: datetime=None):
        response = self.__request_wrapper.make_request(COUNTRIES_METHOD, last_updated=last_updated)
        return Countries(response)

    def regions(self, last_updated: datetime=None):
        response = self.__request_wrapper.make_request(REGIONS_METHOD, last_updated=last_updated)
        return Regions(response)


class TransportNodes(object):
    def __init__(self, json_data):
        self.transport_nodes = get_array(json_data.get('TransportNodes'), TransportNode)

        self.json_data = json_data


class Cities(object):
    def __init__(self, json_data):
        self.cities = get_array(json_data.get('Cities'), City)

        self.json_data = json_data


class Countries(object):
    def __init__(self, json_data):
        self.countries = get_array(json_data.get('Countries'), Country)

        self.json_data = json_data


class Regions(object):
    def __init__(self, json_data):
        self.regions = get_array(json_data.get('Regions'), Region)

        self.json_data = json_data



class ReturnTarget:
    RETURN = 'Return'
    EXCHANGE = 'Exchange'


class SpecialPlacesDemand:
    NO_VALUE = 'NoValue'
    INVALID = 'PlacesForDisabledPersons'
    VIP = 'PlacesForVipPersons'


class TariffType:
    STANDART = 'Standard'
    ROUND_TRIP = 'RoundTrip'
    BUSINESS = 'Business'
    CHILD = 'Child'


class RailwayTransitPermissionApprovalStatus:
    NO_VALUE = 'NoValue'
    ACCEPTED = 'Accepted'
    REJECTED = 'Rejected'


class MealTime:
    BREAKFAST = 'Breakfast'
    LUNCH = 'Lunch'
    DINNER = 'Dinner'


class ProlongReservationType:
    RAILWAY_THREE_HOURS_RESERVATION = 'RailwayThreeHoursReservation'


class PendingElectronicRegistration:
    NO_VALUE = 'NoValue'
    TO_CANCEL = 'ToCancel'


class CarGrouping:
    GROUP = 'Group'
    DONT_GROUP = 'DontGroup'


class PricingTariffType:
    FULL = 'Full'
    CHILD = 'Child'


class RzhdCardTypes:
    RZHD_BONUS = 'RzhdBonus'
    UNIVERSAL_RZHD_CARD = 'UniversalRzhdCard'


class RailwayServiceType:
    TICKETS = 'Tickets'
    TALONS = 'Talons'
    CRIMEA_TALONS = 'CrimeaTalons'


class SimpleOperationStatus:
    ERROR = 'Error'
    IN_PROCESS = 'InProcess'
    SUCCEEDED = 'Succeeded'


class OperationType:
    PURCHASE = 'Purchase'
    RETURN = 'Return'
    EXCHANGE = 'Exchange'


class TransportType:
    TRAIN = 'Train'
    BUS = 'Bus'
    FERRY = 'Ferry'


class PlaceReservationType:
    USUAL = 'Usual'
    TWO_PLACE_AT_ONCE = 'TwoPlacesAtOnce'
    FOUR_PLACE_AT_ONCE = 'FourPlacesAtOnce'


class BookingSystem:
    EXPRESS_3 = 'Express3'
    EXPRESS_2 = 'Express2'
    TEEM = 'Teem'


class RoutePolicy:
    INTERNAL = 'Internal'
    FINLAND = 'Finland'
    INTERNATIONAL = 'International'


class Sex:
    NOVALUE = 'NoValue'
    MALE = 'Male'
    FEMALE = 'Female'


class ProviderPaymentForm:
    CASH = 'Cash'
    INVOICE = 'Invoice'
    CARD = 'Card'


class CarStorey:
    NO_VALUE = 'NoValue'
    FIRST = 'First'
    SECOND = 'Second'


class RailwayPassengerCategory:
    ADULT = 'Adult'
    CHILD = 'Child'
    BABY_WITHOUT_PLACE = 'BabyWithoutPlace'


class RailwayTravelEvent:
    INJURY = 'Injury'
    DEATH = 'Death'
    DISABILITY = 'Disability'


class TransportNodeType:
    AIRPORT = 'Airport'
    RAILWAY_STATION = 'RailwayStation'
    BUS_STOP = 'BusStop'
    AEROEXPRESS_STATION = 'AeroexpressStation'


class StationTimeDescription:
    NO_VALUE = 'NoValue'
    MOSCOW = 'Moscow'
    LOCAL = 'Local'
    GREENWICH = 'Greenwich'


class RouteStopType:
    NO_VALUE = 'NoValue'
    DEPARTURE = 'Departure'
    ARRIVAL = 'Arrival'
    INTERMEDIATE = 'Intermediate'


class CabinPlaceDemands:
    NO_VALUE = 'NoValue'
    IN_ONE_CABIN = 'InOneCabin'
    NO_SIDE_PLACES = 'NoSidePlaces'
    IN_ONE_COMPARTMENT = 'InOneCompartment'


class CabinGenderKind:
    NO_VALUE = 'NoValue'
    MIXED = 'Mixed'
    MALE = 'Male'
    FEMALE = 'Female'


class InsuranceSupplier:
    IGS = 'Igs'
    SOGAZ = 'Sogaz'
    RENESSANS = 'Renessans'
    ROSGOSSTRAH = 'Rosgosstrah'
    BLAGOSOSTOYANIE = 'Blagosostoyanie'


class PassengerSpecifyingRule:
    Standard = 'Standard'
    HELSINKI_WARSAW_CLASS11 = 'HelsinkiWarsawWithClass11'
    HELSINKI_WARSAW_CLASS12 = 'HelsinkiWarsawWithClass12'
    BERLIN_CLASS11 = 'BerlinWithClass11'
    BERLIN_CLASS12 = 'BerlinWithClass12'
    PARIS_PRAGUE_NIVE_CLASS11 = 'ParisPragueNiceWithClass11'
    PARIS_PRAGUE_NIVE_CLASS12 = 'ParisPragueNiceWithClass12'
    TWO_PLACE_AT_ONCE = 'TwoPlacesAtOnce'
    TWO_PLACE_AT_ONCE__ADDITIONAL_CHILD = 'TwoPlacesAtOnceWithAdditionalChild'
    FOUR_PLACE_AT_ONCE = 'FourPlacesAtOnce'
    KUPEK_TARIFF = 'KupekTariff'


class CarType:
    UNKNOWN = 'Unknown'
    SHARED = 'Shared'
    SOFT = 'Soft'
    LUXURY = 'Luxury'
    COMPARTMENT = 'Compartment'
    RESERVED_SEAT = 'ReservedSeat'
    SEDENTARY = 'Sedentary'


class BlankStatus:
    ELECTRONIC_REGISTRATION_ABSENT = 'ElectronicRegistrationAbsent'
    ELECTRONIC_REGISTRATION_PRESENT = 'ElectronicRegistrationPresent'
    NOT_CONFIRMED = 'NotConfirmed'
    VOIDED = 'Voided'
    RETURNED = 'Returned'
    PLACES_RETURNED = 'PlacesReturned'
    VOUCHER_ISSUED = 'VoucherIssued'


class DetailedOperationStatus:
    ERROR = 'Error'
    CREATED = 'Created'
    RESERVED = 'Reserved'
    IN_PROCESS_OF_CANCELLING = 'InProcessOfCancelling'
    IN_PROCESS_OF_FINALIZING = 'InProcessOfFinalizing'
    IN_PROCESS_OF_VOIDING = 'InProcessOfVoiding'
    IN_PROCESS = 'InProcess'
    SUCCEEDED = 'Succeeded'


class DiscountType:
    NO_VALUE = 'NoValue'
    SENIOR = 'Senior'
    JUNIOR = 'Junior'
    ROUND_TRIP = 'RoundTrip'
    ROUND_TRIP_IN_INTERSTATE_DIRECTION = 'RoundTripInInterstateDirection'
    PUPIL = 'Pupil'
    UNIVERSAL_CARD = 'UniversalCard'
    HOLIDAY = 'Holiday'
    WEDDING = 'Wedding'
    FAMILY = 'Family'
    KUPEK = 'Kupek'
    SINGLE = 'Single'


class AdditionalPlaceRequirements:
    NO_VALUE = 'NoValue'
    MOTHER_AND_BABY_PLACES = 'MotherAndBabyPlaces'
    WITH_BABY_PLACES = 'WithBabyPlaces'
    WITH_PETS_PLACES = 'WithPetsPlaces'
    USUAL = 'Usual'
    USUAL_NEAR_THE_TABLE = 'UsualNearTheTable'
    ANY_NEAR_THE_TABLE = 'AnyNearTheTable'
    ANY_NOT_NEAR_THE_TABLE = 'AnyNotNearTheTable'
    NEAR_THE_PLAYGROUND = 'NearThePlayground'
    NEAR_THE_PLAYGROUND_AND_NOT_THE_TABLE = 'NearThePlaygroundAndNotTheTable'
    NEAR_THE_PLAYGROUND_AND_THE_TABLE = 'NearThePlaygroundAndTheTable'
    NEAR_THE_PLACES_WITH_PETS = 'NearThePlacesWithPets'
    FOLDABLE_PLACE = 'FoldablePlace'


class DocumentType:
    RUSSIAN_PASSPORT = 'RussianPassport'
    RUSSIAN_FOREIGN_PASSPORT = 'RussianForeignPassport'
    FOREIGN_PASSPORT = 'ForeignPassport'
    BIRTH_CERTIFICATE = 'BirthCertificate'
    MILITARY_CARD = 'MilitaryCard'
    MILITARY_OFFICER_CARD = 'MilitaryOfficerCard'
    RETURN_TO_CIS_CERTIFICATE = 'ReturnToCisCertificate'
    DIPLOMATIC_PASSPORT = 'DiplomaticPassport'
    SERVICE_PASSPORT = 'ServicePassport'
    SAILOR_PASSPORT = 'SailorPassport'
    STATELESS_PERSON_IDENTITY_CARD = 'StatelessPersonIdentityCard'
    RESIDENCE_PERMIT = 'ResidencePermit'
    RUSSIAN_TEMPORARY_IDENTITY_CARD = 'RussianTemporaryIdentityCard'


class PreferredAdultTariffType:
    FULL = 'Full'
    JUNIOR = 'Junior'
    SENIOR = 'Senior'
    PUPIL = 'Pupil'
    HOLIDAY = 'Holiday'
    WEDDING = 'Wedding'
    FAMILY = 'Family'
    KUPEK = 'Kupek'
    SINGLE = 'Single'
    TKS_P = 'TksP'
    TKS_M = 'TksM'
    TKS_I = 'TksI'


class CarService:
    BEDCLOTHES = 'Bedclothes'
    MEAL = 'Meal'
    WIFI = 'Wifi'
    TV = 'Tv'
    HYGIENIC_KIT = 'HygienicKit'
    PRESS = 'Press'
    AIR_CONDITIONING = 'AirConditioning'
    BIO_TOILET = 'BioToilet'
    PLAID = 'Plaid'
    WASHBASIN_IN_COMPARTMENT = 'WashbasinInCompartment'
    SHOWE_RROOM_IN_COMPARTMENT = 'ShowerRoomInCompartment'
    HYGIENIC_SHOWER = 'HygienicShower'
    SOCKET_220V = 'Socket220V'
    SLIPPERS = 'Slippers'
    PETS_CARRIAGE = 'PetsCarriage'
    PLACES_FOR_PASSENGER_WITH_BABY = 'PlacesForPassengerWithBaby'
    TRANSFER = 'Transfer'


class TicketTariffType:
    FULL = 'Full'
    CHILD = 'Child'
    CHILD17 = 'Child17'
    FREE_CHILD = 'FreeChild'
    JUNIOR = 'Junior'
    SENIOR = 'Senior'
    BACKWARD_WAY_IN_ROUND_TRIP_WITH_DISCOUNT = 'BackwardWayInRoundTripWithDiscount'
    BACKWARD_WAY_IN_ROUND_TRIP_WITH_DISCOUNT_IN_INTERSTATE_DIRECTION = 'BackwardWayInRoundTripWithDiscountInInterstateDirection'
    UNIVERSAL_CARD = 'UniversalCard'
    PUPIL = 'Pupil'
    HOLIDAY = 'Holiday'
    WEDDING = 'Wedding'
    FAMILY = 'Family'
    KUPEK = 'Kupek'
    SINGLE = 'Single'
    TKS = 'Tks'
    CHILD_BACKWARD_WAY_IN_ROUND_TRIP_WITH_DISCOUNT = 'ChildBackwardWayInRoundTripWithDiscount'
    CHILD_BACKWARD_WAY_IN_ROUND_TRIP_WITH_DISCOUNT_IN_INTERSTATE_DIRECTION = 'ChildBackwardWayInRoundTripWithDiscountInInterstateDirection'
    BIRTHDAY = 'Birthday'
    UNKNOWN = 'Unknown'


class ReservationPlaceType:
    NO_VALUE = 'NoValue'
    NEAR_PLAYGROUND = 'NearPlayground'
    UPPER = 'Upper'
    NOT_NEAR_TABLE = 'NotNearTable'
    LOWER = 'Lower'
    MIDDLE = 'Middle'
    FOLDABLE = 'Foldable'
    WITH_PETS = 'WithPets'
    WITH_CHILD = 'WithChild'
    MOTHER_AND_BABY = 'MotherAndBaby'
    NEAR_PASSENGERS_WITH_PETS = 'NearPassengersWithPets'
    NEAR_TABLE = 'NearTable'
    NEAR_TABLE_PLAYGROUND = 'NearTablePlayground'
    SEPARATE_COMPARTMENT = 'SeparateCompartment'
    INVALIDS = 'Invalids'
    NEGOTIATIONS = 'Negotiations'


class CarPlaceType:
    NOVALUE = 'NoValue'
    SEPARATE_COMPARTMENT = 'SeparateCompartment'
    SIDE_LOWER_NEAR_RESTROOM = 'SideLowerNearRestroom'
    SIDE_UPPER_NEAR_RESTROOM = 'SideUpperNearRestroom'
    THIRD_SHELF = 'ThirdShelf'
    FOLDABLE = 'Foldable'
    UPPER = 'Upper'
    LOWER = 'Lower'
    USUAL = 'Usual'
    LAST_KUPE_LOWER = 'LastKupeLower'
    LAST_KUPE_UPPER = 'LastKupeUpper'
    MIDDLE = 'Middle'
    WITH_BICYCLE = 'WithBicycle'
    SIDE_LOWER = 'SideLower'
    SIDE_UPPER = 'SideUpper'
    NEAR_PLAYGROUND = 'NearPlayground'
    NEAR_TABLE_PLAYGROUND = 'NearTablePlayground'
    NEAR_TABLE = 'NearTable'
    WITH_PETS = 'WithPets'
    MOTHER_AND_BABY = 'MotherAndBaby'
    WITH_CHILD = 'WithChild'
    NEAR_PASSENGER_SWITH_PETS = 'NearPassengersWithPets'
    INVALIDS_LOWER = 'InvalidsLower'
    INVALIDS_UPPER = 'InvalidsUpper'
    NEGOTIATIONS = 'Negotiations'



from ..utils import get_array, get_item, get_datetime, get_datetime_array, get_bool_item, get_array_from_str, get_money


class Discount(object):
    def __init__(self, json_data):
        self.discount_type = json_data.get('DiscountType')
        self.description = json_data.get('Description')

        self.json_data = json_data


class FeeCalculation(object):
    def __init__(self, json_data):
        self.charge = get_money(json_data.get('Charge'))
        self.profit = get_money(json_data.get('Profit'))

        self.json_data = json_data


class StationClarifying(object):
    def __init__(self, json_data):
        self.station_type = json_data.get('StationType')
        self.station_options = get_array(json_data.get('StationOptions'), StationInfo)

        self.json_data = json_data


class StationInfo(object):
    def __init__(self, json_data):
        self.station_name = json_data.get('StationName')
        self.station_code = json_data.get('StationCode')

        self.json_data = json_data


class TrainPriceInfo(object):
    def __init__(self, json_data):
        self.has_electronic_registration = get_bool_item(json_data.get('HasElectronicRegistration'))
        self.has_dynamic_pricing_cars = get_bool_item(json_data.get('HasDynamicPricingCars'))
        self.has_two_storey_cars = get_bool_item(json_data.get('HasTwoStoreyCars'))
        self.carriers = json_data.get('Carriers')
        self.car_groups = get_array(json_data.get('CarGroups'), CarGroupPriceInfo)
        self.id = get_item(json_data.get('Id'), int)
        self.train_number = json_data.get('TrainNumber')
        self.train_number_to_get_route = json_data.get('TrainNumberToGetRoute')
        self.display_train_number = json_data.get('DisplayTrainNumber')
        self.train_description = json_data.get('TrainDescription')
        self.train_name = json_data.get('TrainName')
        self.transport_type = json_data.get('TransportType')
        self.origin_name = json_data.get('OriginName')
        self.origin_station_code = json_data.get('OriginStationCode')
        self.destination_name = json_data.get('DestinationName')
        self.destination_station_code = json_data.get('DestinationStationCode')
        self.destination_names = json_data.get('DestinationNames')
        self.local_arrival_date_time = get_datetime(json_data.get('LocalArrivalDateTime'))
        self.arrival_date_time = get_datetime(json_data.get('ArrivalDateTime'))
        self.local_arrival_date_times = get_datetime_array(json_data.get('LocalArrivalDateTimes'))
        self.arrival_date_times = get_datetime_array(json_data.get('ArrivalDateTimes'))
        self.local_departure_date_time = get_datetime(json_data.get('LocalDepartureDateTime'))
        self.departure_date_time = get_datetime(json_data.get('DepartureDateTime'))
        self.departure_date_from_forming_station = get_datetime(json_data.get('DepartureDateFromFormingStation'))
        self.departure_stop_time = get_item(json_data.get('DepartureStopTime'), int)
        self.arrival_stop_time = get_item(json_data.get('ArrivalStopTime'), int)
        self.trip_duration = get_item(json_data.get('TripDuration'), int)
        self.trip_distance = get_item(json_data.get('TripDistance'), int)
        self.is_suburban = get_bool_item(json_data.get('IsSuburban'))
        self.is_component = get_bool_item(json_data.get('IsComponent'))
        self.car_services = json_data.get('CarServices')
        self.is_sale_forbidden = get_bool_item(json_data.get('IsSaleForbidden'))

        self.json_data = json_data


class CarGroupPriceInfo(object):
    def __init__(self, json_data):
        self.car_type = json_data.get('CarType')
        self.car_type_name = json_data.get('CarTypeName')
        self.place_quantity = get_item(json_data.get('PlaceQuantity'), int)
        self.lower_place_quantity = get_item(json_data.get('LowerPlaceQuantity'), int)
        self.upper_place_quantity = get_item(json_data.get('UpperPlaceQuantity'), int)
        self.lower_side_place_quantity = get_item(json_data.get('LowerSidePlaceQuantity'), int)
        self.upper_side_place_quantity = get_item(json_data.get('UpperSidePlaceQuantity'), int)
        self.male_place_quantity = get_item(json_data.get('MalePlaceQuantity'), int)
        self.female_place_quantity = get_item(json_data.get('FemalePlaceQuantity'), int)
        self.empty_cabin_quantity = get_item(json_data.get('EmptyCabinQuantity'), int)
        self.mixed_cabin_quantity = get_item(json_data.get('MixedCabinQuantity'), int)
        self.min_price = get_money(json_data.get('MinPrice'))
        self.max_price = get_money(json_data.get('MaxPrice'))
        self.carriers = json_data.get('Carriers')
        self.car_descriptions = json_data.get('CarDescriptions')
        self.service_classes = json_data.get('ServiceClasses')
        self.service_costs = get_array(json_data.get('ServiceCosts'), get_money)
        self.international_service_classes = json_data.get('InternationalServiceClasses')
        self.availability_indication = json_data.get('AvailabilityIndication')
        self.is_three_hours_reservation_available = get_bool_item(json_data.get('IsThreeHoursReservationAvailable'))
        self.is_deferred_payment_available = get_bool_item(json_data.get('IsDeferredPaymentAvailable'))
        self.is_meal_option_possible = get_bool_item(json_data.get('IsMealOptionPossible'))
        self.is_additional_meal_option_possible = get_bool_item(json_data.get('IsAdditionalMealOptionPossible'))
        self.is_transit_document_required = get_bool_item(json_data.get('IsTransitDocumentRequired'))
        self.is_bedding_selection_possible = get_bool_item(json_data.get('IsBeddingSelectionPossible'))
        self.is_sale_forbidden = get_bool_item(json_data.get('IsSaleForbidden'))
        self.has_electronic_registration = get_bool_item(json_data.get('HasElectronicRegistration'))
        self.has_gender_cabins = get_bool_item(json_data.get('HasGenderCabins'))
        self.has_place_numeration = get_bool_item(json_data.get('HasPlaceNumeration'))
        self.has_places_near_playground = get_bool_item(json_data.get('HasPlacesNearPlayground'))
        self.has_places_near_pets = get_bool_item(json_data.get('HasPlacesNearPets'))
        self.has_places_near_babies = get_bool_item(json_data.get('HasPlacesNearBabies'))
        self.discounts = get_array(json_data.get('Discounts'), Discount)
        self.info_request_schema = json_data.get('InfoRequestSchema')
        self.total_place_quantity = get_item(json_data.get('TotalPlaceQuantity'), int)
        self.place_reservation_types = json_data.get('PlaceReservationTypes')
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)

        self.json_data = json_data


class FreePlacesByCompartments(object):
    def __init__(self, json_data):
        self.compartment_number = json_data.get('CompartmentNumber')
        self.places = get_array_from_str(json_data.get('Places'))


class CarPriceInfo(object):
    def __init__(self, json_data):
        self.car_type = json_data.get('CarType')
        self.car_sub_type = json_data.get('CarSubType')
        self.car_type_name = json_data.get('CarTypeName')
        self.car_number = json_data.get('CarNumber')
        self.service_class = json_data.get('ServiceClass')
        self.international_service_class = json_data.get('InternationalServiceClass')
        self.car_description = json_data.get('CarDescription')
        self.service_class_transcript = json_data.get('ServiceClassTranscript')
        self.free_places = get_array_from_str(json_data.get('FreePlaces'))
        self.place_quantity = get_item(json_data.get('PlaceQuantity'), int)
        self.is_two_storey = json_data.get('IsTwoStorey')
        self.services = json_data.get('Services')
        self.min_price = get_money(json_data.get('MinPrice'))
        self.max_price = get_money(json_data.get('MaxPrice'))
        self.service_cost = get_money(json_data.get('ServiceCost'))
        self.free_places_by_compartments = get_array(json_data.get('FreePlacesByCompartments'), FreePlacesByCompartments)
        self.place_reservation_type = json_data.get('PlaceReservationType')
        self.availability_indication = json_data.get('AvailabilityIndication')
        self.carrier = json_data.get('Carrier')
        self.has_gender_cabins = json_data.get('HasGenderCabins')
        self.rzhd_card_types = json_data.get('RzhdCardTypes')
        self.train_number = json_data.get('TrainNumber')
        self.local_arrival_date_time = get_datetime(json_data.get('LocalArrivalDateTime'))
        self.arrival_date_time = get_datetime(json_data.get('ArrivalDateTime'))
        self.has_no_interchange = get_bool_item(json_data.get('HasNoInterchange'))
        self.has_place_numeration = get_bool_item(json_data.get('HasPlaceNumeration'))
        self.is_bedding_selection_possible = get_bool_item(json_data.get('IsBeddingSelectionPossible'))
        self.has_electronic_registration = get_bool_item(json_data.get('HasElectronicRegistration'))
        self.has_dynamic_pricing = get_bool_item(json_data.get('HasDynamicPricing'))
        self.has_places_near_babies = get_bool_item(json_data.get('HasPlacesNearBabies'))
        self.has_places_near_playground = get_bool_item(json_data.get('HasPlacesNearPlayground'))
        self.has_places_near_pets = get_bool_item(json_data.get('HasPlacesNearPets'))
        self.is_additional_passenger_allowed = get_bool_item(json_data.get('IsAdditionalPassengerAllowed'))
        self.is_meal_option_possible = get_bool_item(json_data.get('IsMealOptionPossible'))
        self.is_additional_meal_option_possible = get_bool_item(json_data.get('IsAdditionalMealOptionPossible'))
        self.is_transit_document_required = get_bool_item(json_data.get('IsTransitDocumentRequired'))
        self.is_child_tariff_type_allowed = get_bool_item(json_data.get('IsChildTariffTypeAllowed'))
        self.car_place_type = json_data.get('CarPlaceType')
        self.discounts = get_array(json_data.get('Discounts'), Discount)
        self.is_sale_forbidden = get_bool_item(json_data.get('IsSaleForbidden'))
        self.is_three_hours_reservation_available = get_bool_item(json_data.get('IsThreeHoursReservationAvailable'))
        self.road = json_data.get('Road')
        self.passenger_specifying_rules = json_data.get('PassengerSpecifyingRules')
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)

        self.json_data = json_data


class TrainInfo(object):
    def __init__(self, json_data):
        self.train_number = json_data.get('TrainNumber')
        self.train_number_to_get_route = json_data.get('TrainNumberToGetRoute')
        self.display_train_number = json_data.get('DisplayTrainNumber')
        self.train_description = json_data.get('TrainDescription')
        self.train_name = json_data.get('TrainName')
        self.transport_type = json_data.get('TransportType')
        self.origin_name = json_data.get('OriginName')
        self.origin_station_code = json_data.get('OriginStationCode')
        self.destination_name = json_data.get('DestinationName')
        self.destination_station_code = json_data.get('DestinationStationCode')
        self.destination_names = json_data.get('DestinationNames')
        self.local_departure_date_time = get_datetime(json_data.get('LocalDepartureDateTime'))
        self.departure_date_time = get_datetime(json_data.get('DepartureDateTime'))
        self.local_arrival_date_time = get_datetime(json_data.get('LocalArrivalDateTime'))
        self.arrival_date_time = get_datetime(json_data.get('ArrivalDateTime'))
        self.local_arrival_date_times = get_datetime_array(json_data.get('LocalArrivalDateTimes'))
        self.arrival_date_times = get_datetime_array(json_data.get('ArrivalDateTimes'))
        self.departure_date_from_forming_station = get_datetime(json_data.get('DepartureDateFromFormingStation'))
        self.departure_stop_time = get_item(json_data.get('DepartureStopTime'), int)
        self.arrival_stop_time = get_item(json_data.get('ArrivalStopTime'), int)
        self.trip_duration = get_item(json_data.get('TripDuration'), int)
        self.trip_distance = get_item(json_data.get('TripDistance'), int)
        self.is_suburban = get_bool_item(json_data.get('IsSuburban'))
        self.is_component = get_bool_item(json_data.get('IsComponent'))
        self.car_services = json_data.get('CarServices')
        self.is_sale_forbidden = get_bool_item(json_data.get('IsSaleForbidden'))

        self.json_data = json_data


class ScheduleInfo(object):
    def __init__(self, json_data):
        self.train_number = json_data.get('TrainNumber')
        self.train_number_to_get_route = json_data.get('TrainNumberToGetRoute')
        self.train_name = json_data.get('TrainName')
        self.origin_name = json_data.get('OriginName')
        self.origin_station_code = json_data.get('OriginStationCode')
        self.destination_name = json_data.get('DestinationName')
        self.destination_station_code = json_data.get('DestinationStationCode')
        self.local_departure_time = json_data.get('LocalDepartureTime')
        self.departure_time = json_data.get('DepartureTime')
        self.departure_stop_time = get_item(json_data.get('DepartureStopTime'), int)
        self.local_arrival_time = json_data.get('LocalArrivalTime')
        self.arrival_time = json_data.get('ArrivalTime')
        self.arrival_stop_time = get_item(json_data.get('ArrivalStopTime'), int)
        self.trip_duration = get_item(json_data.get('TripDuration'), int)
        self.trip_distance = get_item(json_data.get('TripDistance'), int)
        self.regularity = json_data.get('Regularity')
        self.start_sales_date_time = get_datetime(json_data.get('StartSalesDateTime'))

        self.json_data = json_data


class TrainRouteRoute(object):
    def __init__(self, json_data):
        self.name = json_data.get('Name')
        self.origin_name = json_data.get('OriginName')
        self.destination_name = json_data.get('DestinationName')
        self.route_stops = get_array(json_data.get('RouteStops'), RouteStopInfo)

        self.json_data = json_data


class RouteStopInfo(object):
    def __init__(self, json_data):
        self.station_name = json_data.get('StationName')
        self.city_name = json_data.get('CityName')
        self.local_departure_time = json_data.get('LocalDepartureTime')
        self.departure_time = json_data.get('DepartureTime')
        self.arrival_time = json_data.get('ArrivalTime')
        self.local_arrival_times = json_data.get('LocalArrivalTimes')
        self.arrival_times = json_data.get('ArrivalTimes')
        self.local_arrival_time = json_data.get('LocalArrivalTime')
        self.route_stop_type = json_data.get('RouteStopType')
        self.stop_duration = get_item(json_data.get('StopDuration'), int)
        self.time_description = json_data.get('TimeDescription')
        self.station_time_description = json_data.get('StationTimeDescription')

        self.json_data = json_data


class Route(object):
    def __init__(self, json_data):
        self.origin_code = json_data.get('OriginCode')
        self.origin_station_code = json_data.get('OriginStationCode')
        self.destination_code = json_data.get('DestinationCode')
        self.destination_station_code = json_data.get('DestinationStationCode')
        self.departure_date_time = get_datetime(json_data.get('DepartureDateTime'))
        self.arrival_date_time = get_datetime(json_data.get('ArrivalDateTime'))
        self.travel_time = get_item(json_data.get('TravelTime'), int)
        self.travel_distance = get_item(json_data.get('TravelDistance'), int)
        self.change_time = get_item(json_data.get('ChangeTime'), int)
        self.route_parts = get_array(json_data.get('RouteParts'), RoutePart)

        self.json_data = json_data


class RoutePart(object):
    def __init__(self, json_data):
        self.train_number = json_data.get('TrainNumber')
        self.origin_city_code = json_data.get('OriginCityCode')
        self.origin_station_code = json_data.get('OriginStationCode')
        self.destination_city_code = json_data.get('DestinationCityCode')
        self.destination_station_code = json_data.get('DestinationStationCode')
        self.departure_date_time = get_datetime(json_data.get('DepartureDateTime'))
        self.arrival_date_time = get_datetime(json_data.get('ArrivalDateTime'))

        self.json_data = json_data


class TrainPricingResponse(object):
    def __init__(self, json_data):
        self.origin_code = json_data.get('OriginCode')
        self.origin_station_code = json_data.get('OriginStationCode')
        self.origin_time_zone_difference = get_item(json_data.get('OriginTimeZoneDifference'), int)
        self.destination_code = json_data.get('DestinationCode')
        self.destination_station_code = json_data.get('DestinationStationCode')
        self.destination_time_zone_difference = get_item(json_data.get('DestinationTimeZoneDifference'), int)
        self.trains = get_array(json_data.get('Trains'), TrainPriceInfo)
        self.departure_time_description = json_data.get('DepartureTimeDescription')
        self.arrival_time_description = json_data.get('ArrivalTimeDescription')
        self.is_from_ukrain = get_bool_item(json_data.get('IsFromUkrain'))
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)
        self.not_all_trains_returned = get_bool_item(json_data.get('NotAllTrainsReturned'))
        self.station_clarifying = get_item(json_data.get('StationClarifying'), StationClarifying)
        self.booking_system = json_data.get('BookingSystem')
        self.id = get_item(json_data.get('Id'), int)
        self.route_policy = json_data.get('RoutePolicy')

        self.json_data = json_data


class RouteReferenced(object):
    def __init__(self, json_data):
        self.route_parts = get_array(json_data.get('RouteParts'), RoutePartReferenced)

        self.json_data = json_data


class RoutePartReferenced(object):
    def __init__(self, json_data):
        self.pricing_id = get_item(json_data.get('PricingId'), int)
        self.train_id = get_item(json_data.get('TrainId'), int)

        self.json_data = json_data


class AdditionalMealOption(object):
    def __init__(self, json_data):
        self.amount = get_money(json_data.get('Amount'))
        self.price_per_unit = get_money(json_data.get('PricePerUnit'))
        self.quantity = get_item(json_data.get('Quantity'), int)
        self.meal_time = json_data.get('MealTime')
        self.meal_name = json_data.get('MealName')
        self.meal_option_code = json_data.get('MealOptionCode')
        self.description = json_data.get('Description')

        self.json_data = json_data


class MealOption(object):
    def __init__(self, json_data):
        self.meal_option_code = json_data.get('MealOptionCode')
        self.description = json_data.get('Description')
        self.meal_type = json_data.get('MealType')

        self.json_data = json_data


class PlaceWithType(object):
    def __init__(self, json_data):
        self.number = json_data.get('Number')
        self.type = json_data.get('Type')

        self.json_data = json_data


class PassengerResponse(object):
    def __init__(self, json_data):
        self.category = json_data.get('Category')
        self.places = get_array_from_str(json_data.get('Places'))
        self.place_tiers = get_array_from_str(json_data.get('PlaceTiers'))
        self.places_with_type = get_array(json_data.get('PlacesWithType'), PlaceWithType)
        self.tariff_type = json_data.get('TariffType')
        self.first_name = json_data.get('FirstName')
        self.middle_name = json_data.get('MiddleName')
        self.last_name = json_data.get('LastName')
        self.birthday = get_datetime(json_data.get('BirthDay'))
        self.sex = json_data.get('Sex')
        self.citizenship = json_data.get('Citizenship')
        self.order_item_blank_id = get_item(json_data.get('OrderItemBlankId'), int)
        self.order_customer_id = get_item(json_data.get('OrderCustomerId'), int)
        self.order_customer_reference_index = get_item(json_data.get('OrderCustomerReferenceIndex'), int)
        self.amount = get_money(json_data.get('Amount'))
        self.fare = get_money(json_data.get('Fare'))
        self.tax = get_money(json_data.get('Tax'))
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)

        self.json_data = json_data


class RateValue(object):
    def __init__(self, json_data):
        self.rate = get_item(json_data.get('Rate'), float)
        self.value = get_money(json_data.get('Value'))

        self.json_data = json_data


class TicketTariffInfo(object):
    def __init__(self, json_data):
        self.tariff_type = json_data.get('TariffType')
        self.tariff_name = json_data.get('TariffName')

        self.json_data = json_data


class PrepaidMealInfo(object):
    def __init__(self, json_data):
        self.meal_option_code = json_data.get('MealOptionCode')
        self.meal_name = json_data.get('MealName')
        self.description = json_data.get('Description')

        self.json_data = json_data


class ReservationBlankResponse(object):
    def __init__(self, json_data):
        self.order_item_blank_id = get_item(json_data.get('OrderItemBlankId'), int)
        self.amount = get_money(json_data.get('Amount'))
        self.number = json_data.get('Number')
        self.tariff_type = json_data.get('TariffType')
        self.is_meal_option_possible = get_bool_item(json_data.get('IsMealOptionPossible'))
        self.vat_rate_values = get_array(json_data.get('VatRateValues'), RateValue)
        self.base_fare = get_money(json_data.get('BaseFare'))
        self.additional_price = get_money(json_data.get('AdditionalPrice'))
        self.tariff_info = get_item(json_data.get('TariffInfo'), TicketTariffInfo)
        self.tariff_additional_info = json_data.get('TariffAdditionalInfo')
        self.prepaid_meal_info = get_item(json_data.get('PrepaidMealInfo'), PrepaidMealInfo)
        self.service_price = get_money(json_data.get('ServicePrice'))

        self.json_data = json_data


class ReservationResponse(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.departure_date_time = get_datetime(json_data.get('DepartureDateTime'))
        self.arrival_date_time = get_datetime(json_data.get('ArrivalDateTime'))
        self.origin_station = json_data.get('OriginStation')
        self.origin_station_code = json_data.get('OriginStationCode')
        self.origin_time_zone_difference = get_item(json_data.get('OriginTimeZoneDifference'), int)
        self.destination_station = json_data.get('DestinationStation')
        self.destination_station_code = json_data.get('DestinationStationCode')
        self.destination_time_zone_difference = get_item(json_data.get('DestinationTimeZoneDifference'), int)
        self.blanks = get_array(json_data.get('Blanks'), ReservationBlankResponse)
        self.train_number = json_data.get('TrainNumber')
        self.booking_train_number = json_data.get('BookingTrainNumber')
        self.train_number_to_get_route = json_data.get('TrainNumberToGetRoute')
        self.car_number = json_data.get('CarNumber')
        self.car_type = json_data.get('CarType')
        self.lower_place_quantity = json_data.get('LowerPlaceQuantity')
        self.upper_place_quantity = json_data.get('UpperPlaceQuantity')
        self.service_class = json_data.get('ServiceClass')
        self.international_service_class = json_data.get('InternationalServiceClass')
        self.time_description = json_data.get('TimeDescription')
        self.is_print_ticket_possible = get_bool_item(json_data.get('IsPrintTicketPossible'))
        self.nearest_ticket_print_stations = json_data.get('NearestTicketPrintStations')
        self.carrier = json_data.get('Carrier')
        self.carrier_code = json_data.get('CarrierCode')
        self.carrier_tin = json_data.get('CarrierTin')
        self.country_code = json_data.get('CountryCode')
        self.is_meal_option_possible = get_bool_item(json_data.get('IsMealOptionPossible'))
        self.is_additional_meal_option_possible = get_bool_item(json_data.get('IsAdditionalMealOptionPossible'))
        self.meal_group = json_data.get('MealGroup')
        self.booking_system = json_data.get('BookingSystem')
        self.is_three_hours_reservation_available = json_data.get('IsThreeHoursReservationAvailable')
        self.trip_duration = get_item(json_data.get('TripDuration'), int)
        self.train_description = json_data.get('TrainDescription')
        self.car_description = json_data.get('CarDescription')
        self.is_suburban = get_bool_item(json_data.get('IsSuburban'))
        self.cabin_gender_description = json_data.get('CabinGenderDescription')
        self.is_only_full_return_possible = json_data.get('IsOnlyFullReturnPossible')
        self.passengers = get_array(json_data.get('Passengers'), PassengerResponse)
        self.index = get_item(json_data.get('Index'), int)
        self.order_item_id = get_item(json_data.get('OrderItemId'), int)
        self.agent_reference_id = json_data.get('AgentReferenceId')
        self.amount = get_money(json_data.get('Amount'))
        self.fare = get_money(json_data.get('Fare'))
        self.tax = get_money(json_data.get('Tax'))
        self.reservation_number = json_data.get('ReservationNumber')
        self.confirm_till = get_datetime(json_data.get('ConfirmTill'))
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)
        self.error_result = json_data.get('ErrorResult')

        self.json_data = json_data


class OrderCreateReservationCustomerResponse(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.index = get_item(json_data.get('Index'), int)
        self.order_customer_id = get_item(json_data.get('OrderCustomerId'), int)
        self.first_name = json_data.get('FirstName')
        self.middle_name = json_data.get('MiddleName')
        self.last_name = json_data.get('LastName')
        self.sex = json_data.get('Sex')
        self.birth_date = get_datetime(json_data.get('BirthDate'))
        self.document_number = json_data.get('DocumentNumber')
        self.document_valid_till = get_datetime(json_data.get('DocumentValidTill'))
        self.document_type = json_data.get('DocumentType')
        self.citizenship_code = json_data.get('CitizenshipCode')
        self.birth_place = json_data.get('BirthPlace')
        self.birthday = get_datetime(json_data.get('Birthday'))

        self.json_data = json_data


class OrderCustomerResponse(object):
    def __init__(self, json_data):
        self.order_customer_id = get_item(json_data.get('OrderCustomerId'), int)
        self.first_name = json_data.get('FirstName')
        self.middle_name = json_data.get('MiddleName')
        self.last_name = json_data.get('LastName')
        self.sex = json_data.get('Sex')
        self.birth_date = get_datetime(json_data.get('BirthDate'))
        self.document_number = json_data.get('DocumentNumber')
        self.document_valid_till = get_datetime(json_data.get('DocumentValidTill'))
        self.document_type = json_data.get('DocumentType')
        self.citizenship_code = json_data.get('CitizenshipCode')
        self.birth_place = json_data.get('BirthPlace')

        self.json_data = json_data


class RailwayBlankInfo(object):
    def __init__(self, json_data):
        self.order_item_blank_id = get_item(json_data.get('OrderItemBlankId'), int)
        self.number = json_data.get('Number')
        self.blank_status = json_data.get('BlankStatus')
        self.pending_electronic_registration = json_data.get('PendingElectronicRegistration')
        self.sign_sequence = json_data.get('sign_sequence')

        self.json_data = json_data


class OrderItemCustomerResponseBase(object):
    def __init__(self, json_data):
        self.order_customer_id = get_item(json_data.get('OrderCustomerId'), int)
        self.amount = get_money(json_data.get('Amount'))
        self.fare = get_money(json_data.get('Fare'))
        self.tax = get_money(json_data.get('Tax'))
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)

        self.json_data = json_data


class ApiErrorResult(object):
    def __init__(self, json_data):
        self.code = json_data.get('Code')
        self.message = json_data.get('Message')
        self.message_params = json_data.get('MessageParams')

        self.json_data = json_data


class RailwayConfirmResponse(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.reservation_number = json_data.get('ReservationNumber')
        self.blanks = get_array(json_data.get('Blanks'), RailwayBlankInfo)
        self.expiration_electronic_registration_date_time = get_datetime(json_data.get('ExpirationElectronicRegistrationDateTime'))
        self.order_item_id = get_item(json_data.get('OrderItemId'), int)
        self.amount = get_money(json_data.get('Amount'))
        self.fare = get_money(json_data.get('Fare'))
        self.tax = get_money(json_data.get('Tax'))
        self.confirmed = get_datetime(json_data.get('Confirmed'))
        self.void_till = get_datetime(json_data.get('VoidTill'))
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)
        self.order_item_customers = get_array(json_data.get('OrderItemCustomers'), OrderItemCustomerResponseBase)
        self.warnings = get_array(json_data.get('Warnings'), ApiErrorResult)
        self.error_result = get_item(json_data.get('ErrorResult'), ApiErrorResult)

        self.json_data = json_data


class RailwayReturnBlankResponse(object):
    def __init__(self, json_data):
        self.purchase_order_item_blank_id = get_item(json_data.get('PurchaseOrderItemBlankId'), int)
        self.return_order_item_blank_id = get_item(json_data.get('ReturnOrderItemBlankId'), int)
        self.amount = get_money(json_data.get('Amount'))
        self.penalty = get_money(json_data.get('Penalty'))
        self.vat_rate_values = get_array(json_data.get('VatRateValues'), RateValue)
        self.service_price = get_money(json_data.get('ServicePrice'))

        self.json_data = json_data


class RailwayReturnAmountResponse(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.blanks = get_array(json_data.get('Blanks'), RailwayReturnBlankResponse)
        self.amount = get_money(json_data.get('Amount'))
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)

        self.json_data = json_data


class RailwayAutoReturnResponse(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.blanks = get_array(json_data.get('Blanks'), RailwayReturnBlankResponse)
        self.amount = get_money(json_data.get('Amount'))
        self.fare = get_money(json_data.get('Fare'))
        self.tax = get_money(json_data.get('Tax'))
        self.confirmed = get_datetime(json_data.get('Confirmed'))
        self.return_order_item_id = get_item(json_data.get('ReturnOrderItemId'), int)
        self.agent_reference_id = json_data.get('AgentReferenceId')
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)

        self.json_data = json_data


class OrderCustomerInfo(object):
    def __init__(self, json_data):
        self.order_customer_id = get_item(json_data.get('OrderCustomerId'), int)
        self.first_name = json_data.get('FirstName')
        self.middle_name = json_data.get('MiddleName')
        self.last_name = json_data.get('LastName')
        self.sex = json_data.get('Sex')
        self.birth_date = get_datetime(json_data.get('BirthDate'))
        self.document_number = json_data.get('DocumentNumber')
        self.document_valid_till = get_datetime(json_data.get('DocumentValidTill'))
        self.document_type = json_data.get('DocumentType')
        self.citizenship_code = json_data.get('CitizenshipCode')

        self.json_data = json_data


class RailwayOrderItemCustomerInfo(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.order_item_blank_id = json_data.get('OrderItemBlankId')
        self.places = get_array_from_str(json_data.get('Places'))
        self.place_quantity = json_data.get('PlaceQuantity')
        self.transit_document = json_data.get('TransitDocument')
        self.category = json_data.get('Category')
        self.order_customer_id = json_data.get('OrderCustomerId')
        self.order_item_customer_id = json_data.get('OrderItemCustomerId')
        self.amount = json_data.get('Amount')
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)

        self.json_data = json_data


class RailwayOrderItemBlankInfo(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.voucher_number = json_data.get('VoucherNumber')
        self.base_fare = get_money(json_data.get('BaseFare'))
        self.additional_price = get_money(json_data.get('AdditionalPrice'))
        self.service_price = get_money(json_data.get('ServicePrice'))
        self.vat_rate_values = get_array(json_data.get('VatRateValues'), RateValue)
        self.tariff_type = json_data.get('TariffType')
        self.blank_status = json_data.get('BlankStatus')
        self.is_electronic_registration_set = json_data.get('IsElectronicRegistrationSet')
        self.is_meal_option_possible = json_data.get('IsMealOptionPossible')
        self.pending_electronic_registration = json_data.get('PendingElectronicRegistration')
        self.electronic_registration_set_date_time = get_datetime(json_data.get('ElectronicRegistrationSetDateTime'))
        self.sign_sequence = json_data.get('SignSequence')
        self.tariff_info = get_item(json_data.get('TariffInfo'), TicketTariffInfo)
        self.prepaid_meal_info = get_item(json_data.get('PrepaidMealInfo'), PrepaidMealInfo)
        self.transit_permission_approval_status = json_data.get('TransitPermissionApprovalStatus')
        self.place_quantity = json_data.get('PlaceQuantity')
        self.order_item_blank_id = json_data.get('OrderItemBlankId')
        self.previous_order_item_blank_id = json_data.get('PreviousOrderItemBlankId')
        self.blank_number = json_data.get('BlankNumber')
        self.amount = get_money(json_data.get('Amount'))

        self.json_data = json_data


class RailwayFullOrderItemInfo(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.service_type = json_data.get('ServiceType')
        self.place_quantity = get_item(json_data.get('PlaceQuantity'), int)
        self.origin_station_name = json_data.get('OriginStationName')
        self.origin_time_zone_difference = get_item(json_data.get('OriginTimeZoneDifference'), int)
        self.destination_station_name = json_data.get('DestinationStationName')
        self.destination_time_zone_difference = get_item(json_data.get('DestinationTimeZoneDifference'), int)
        self.train_number = json_data.get('TrainNumber')
        self.booking_train_number = json_data.get('BookingTrainNumber')
        self.train_number_to_get_route = json_data.get('TrainNumberToGetRoute')
        self.car_number = json_data.get('CarNumber')
        self.car_type = json_data.get('CarType')
        self.electronic_registration_expiration_date_time = get_datetime(json_data.get('ElectronicRegistrationExpirationDateTime'))
        self.place_reservation_type = json_data.get('PlaceReservationType')
        self.service_class = json_data.get('ServiceClass')
        self.additional_information = json_data.get('AdditionalInformation')
        self.carrier_description = json_data.get('CarrierDescription')
        self.is_only_full_return_possible = json_data.get('IsOnlyFullReturnPossible')
        self.order_item_customers = get_array(json_data.get('OrderItemCustomers'), RailwayOrderItemCustomerInfo)
        self.order_item_blanks = get_array(json_data.get('OrderItemBlanks'), RailwayOrderItemBlankInfo)
        self.arrival_date_time = get_datetime(json_data.get('ArrivalDateTime'))
        self.local_arrival_date_time = get_datetime(json_data.get('LocalArrivalDateTime'))
        self.origin_location_code = json_data.get('OriginLocationCode')
        self.origin_location_name = json_data.get('OriginLocationName')
        self.destination_location_code = json_data.get('DestinationLocationCode')
        self.destination_location_name = json_data.get('DestinationLocationName')
        self.order_id = get_item(json_data.get('OrderId'), int)
        self.previous_order_item_id = get_item(json_data.get('PreviousOrderItemId'), int)
        self.agent_reference_id = json_data.get('AgentReferenceId')
        self.order_item_id = get_item(json_data.get('OrderItemId'), int)
        self.pos_sys_name = json_data.get('PosSysName')
        self.amount = get_money(json_data.get('Amount'))
        self.reservation_number = json_data.get('ReservationNumber')
        self.operation_type = json_data.get('OperationType')
        self.simple_operation_status = json_data.get('SimpleOperationStatus')
        self.detailed_operation_status = json_data.get('DetailedOperationStatus')
        self.departure_date_time = get_datetime(json_data.get('DepartureDateTime'))
        self.local_departure_date_time = get_datetime(json_data.get('LocalDepartureDateTime'))
        self.create_date_time = get_datetime(json_data.get('CreateDateTime'))
        self.confirm_time_limit = get_datetime(json_data.get('ConfirmTimeLimit'))
        self.confirm_date_time = get_datetime(json_data.get('ConfirmDateTime'))
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)
        self.provider_payment_form = json_data.get('ProviderPaymentForm')
        self.is_externally_loaded = get_bool_item(json_data.get('IsExternallyLoaded'))

        self.json_data = json_data


class RailwayShortOrderItemBlankInfo(object):
    def __init__(self, json_data):
        self.place_quantity = get_item(json_data.get('PlaceQuantity'), int)
        self.order_item_blank_id = get_item(json_data.get('OrderItemBlankId'), int)
        self.previous_order_item_blank_id = get_item(json_data.get('PreviousOrderItemBlankId'), int)
        self.blank_number = json_data.get('BlankNumber')
        self.amount = get_money(json_data.get('Amount'))

        self.json_data = json_data


class RailwayShortOrderItemInfo(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.service_type = json_data.get('ServiceType')
        self.place_quantity = get_item(json_data.get('PlaceQuantity'), int)
        self.order_item_blanks = get_array(json_data.get('OrderItemBlanks'), RailwayShortOrderItemBlankInfo)
        self.order_id = get_item(json_data.get('OrderId'), int)
        self.agent_reference_id = json_data.get('AgentReferenceId')
        self.order_item_id = get_item(json_data.get('OrderItemId'), int)
        self.pos_sys_name = json_data.get('PosSysName')
        self.amount = get_money(json_data.get('Amount'))
        self.reservation_number = json_data.get('ReservationNumber')
        self.operation_type = json_data.get('OperationType')
        self.simple_operation_status = json_data.get('SimpleOperationStatus')
        self.detailed_operation_status = json_data.get('DetailedOperationStatus')
        self.departure_date_time = get_datetime(json_data.get('DepartureDateTime'))
        self.create_date_time = get_datetime(json_data.get('CreateDateTime'))
        self.confirm_time_limit = get_datetime(json_data.get('ConfirmTimeLimit'))
        self.confirm_date_time = get_datetime(json_data.get('ConfirmDateTime'))
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)
        self.provider_payment_form = json_data.get('ProviderPaymentForm')
        self.is_externally_loaded = get_bool_item(json_data.get('IsExternallyLoaded'))

        self.json_data = json_data


class RailwayShortOrderInfo(object):
    def __init__(self, json_data):
        self.order_items = get_array(json_data.get('OrderItems'), RailwayShortOrderItemInfo)
        self.order_id = get_item(json_data.get('OrderId'), int)
        self.amount = get_money(json_data.get('Amount'))
        self.contact_phone = json_data.get('ContactPhone')
        self.contact_emails = json_data.get('ContactEmails')
        self.created = get_datetime(json_data.get('Created'))
        self.confirmed = get_datetime(json_data.get('Confirmed'))
        self.pos_sys_name = json_data.get('PosSysName')

        self.json_data = json_data


class ServiceUpsaleOperationItemResult(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.upsale_order_item_id = get_item(json_data.get('UpsaleOrderItemId'), int)
        self.amount = get_money(json_data.get('Amount'))
        self.is_succeeded = get_bool_item(json_data.get('IsSucceeded'))

        self.json_data = json_data


class CustomerUpsaleOperationResult(object):
    def __init__(self, json_data):
        self.order_customer_id = get_item(json_data.get('OrderCustomerId'), int)
        self.items = get_array(json_data.get('Items'), ServiceUpsaleOperationItemResult)

        self.json_data = json_data


class GeoPoint(object):
    def __init__(self, json_data):
        self.latitude = json_data.get('Latitude')
        self.longitude = json_data.get('Longitude')

        self.json_data = json_data


class TransportNode(object):
    def __init__(self, json_data):
        self.transport_node_id = json_data.get('TransportNodeId')
        self.country_id = json_data.get('CountryId')
        self.region_id = json_data.get('RegionId')
        self.city_id = json_data.get('CityId')
        self.code = json_data.get('Code')
        self.name_ru = json_data.get('NameRu')
        self.name_en = json_data.get('NameEn')
        self.type = json_data.get('Type')
        self.popularity_index = json_data.get('PopularityIndex')
        self.description = json_data.get('Description')
        self.is_active = json_data.get('IsActive')
        self.is_visible = json_data.get('IsVisible')
        self.updated = get_datetime(json_data.get('Updated'))
        self.utc_time_offset = json_data.get('UtcTimeOffset')
        self.location = get_item(json_data.get('Location'), GeoPoint)

        self.json_data = json_data


class City(object):
    def __init__(self, json_data):
        self.city_id = json_data.get('CityId')
        self.country_id = json_data.get('CountryId')
        self.region_id = json_data.get('RegionId')
        self.sys_code = json_data.get('SysCode')
        self.express_code = json_data.get('ExpressCode')
        self.code = json_data.get('Code')
        self.name_ru = json_data.get('NameRu')
        self.name_en = json_data.get('NameEn')
        self.popularity_index = json_data.get('PopularityIndex')
        self.is_active = json_data.get('IsActive')
        self.updated = get_datetime(json_data.get('Updated'))

        self.json_data = json_data


class Country(object):
    def __init__(self, json_data):
        self.country_id = json_data.get('CountryId')
        self.alpha2_code = json_data.get('Alpha2Code')
        self.name_ru = json_data.get('NameRu')
        self.name_en = json_data.get('NameEn')
        self.is_active = json_data.get('IsActive')
        self.updated = get_datetime(json_data.get('Updated'))

        self.json_data = json_data


class Region(object):
    def __init__(self, json_data):
        self.region_id = json_data.get('RegionId')
        self.country_id = json_data.get('CountryId')
        self.iso_code = json_data.get('IsoCode')
        self.name_ru = json_data.get('NameRu')
        self.name_en = json_data.get('NameEn')
        self.is_active = json_data.get('IsActive')
        self.updated = get_datetime(json_data.get('Updated'))

        self.json_data = json_data


class AgentAccount(object):
    def __init__(self, json_data):
        self.current_balance = json_data.get('CurrentBalance')
        self.account_name = json_data.get('AccountName')

        self.json_data = json_data


class RailwayCompensationInfo(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.event = json_data.get('Event')
        self.compensation = json_data.get('Compensation')

        self.json_data = json_data


class RailwayProductPricingInfo(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.compensations = get_array(json_data.get('Compensations'), RailwayCompensationInfo)
        self.package = json_data.get('Package')
        self.compensation = json_data.get('Compensation')
        self.amount = json_data.get('Amount')

        self.json_data = json_data


class RailwayPricingInfo(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.product_pricing_info_list = get_array(json_data.get('ProductPricingInfoList'), RailwayProductPricingInfo)
        self.supplier = json_data.get('Supplier')

        self.json_data = json_data


class RailwayPricingResponse(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.product_pricing_info_list = get_array(json_data.get('ProductPricingInfoList'), RailwayProductPricingInfo)
        self.pricing_info_list = get_array(json_data.get('PricingInfoList'), RailwayPricingInfo)

        self.json_data = json_data


class RaceInfo(object):
    def __init__(self, json_data):
        self.race_id = json_data.get('RaceId')
        self.race_name = json_data.get('RaceName')
        self.free_place_quantity = json_data.get('FreePlaceQuantity')

        self.json_data = json_data


class TariffPriceInfoResponse(object):
    def __init__(self, json_data):
        self.document_types = json_data.get('DocumentTypes')
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)
        self.tariff_id = json_data.get('TariffId')
        self.tariff_name = json_data.get('TariffName')
        self.tariff_type = json_data.get('TariffType')
        self.route_name = json_data.get('RouteName')
        self.description = json_data.get('Description')
        self.price = json_data.get('Price')
        self.max_tickets_quantity_allowed_for_booking = json_data.get('MaxTicketsQuantityAllowedForBooking')
        self.is_for_guaranteed_seats = json_data.get('IsForGuaranteedSeats')
        self.races = get_array(json_data.get('Races'), RaceInfo)


class AeroexpressConfirmResponse(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.reservation_number = json_data.get('ReservationNumber')
        self.return_till = json_data.get('ReturnTill')
        self.order_item_id = json_data.get('OrderItemId')
        self.amount = json_data.get('Amount')
        self.fare = json_data.get('Fare')
        self.tax = json_data.get('Tax')
        self.confirmed = json_data.get('Confirmed')
        self.void_till = json_data.get('VoidTill')
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)
        self.order_item_customers = get_array(json_data.get('OrderItemCustomers'), OrderItemCustomerResponseBase)
        self.warnings = get_array(json_data.get('Warnings'), ApiErrorResult)
        self.error_result = get_item(json_data.get('ErrorResult'), ApiErrorResult)


class AeroexpressReturnBlankResponse(object):
    def __init__(self, json_data):
        self.purchase_order_item_blank_id = json_data.get('PurchaseOrderItemBlankId')
        self.return_order_item_blank_id = json_data.get('ReturnOrderItemBlankId')
        self.amount = json_data.get('Amount')

        self.json_data = json_data


class AeroexpressAutoReturnResponse(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.blanks = get_array(json_data.get('Blanks'), AeroexpressReturnBlankResponse)
        self.amount = json_data.get('Amount')
        self.fare = json_data.get('Fare')
        self.tax = json_data.get('Tax')
        self.confirmed = get_datetime(json_data.get('Confirmed'))
        self.return_order_item_id = json_data.get('ReturnOrderItemId')
        self.agent_reference_id = json_data.get('AgentReferenceId')
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)

        self.json_data = json_data


class AeroexpressOrderItemBlankInfo(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.tariff_type = json_data.get('TariffType')
        self.order_item_blank_id = json_data.get('OrderItemBlankId')
        self.previous_order_item_blank_id = json_data.get('PreviousOrderItemBlankId')
        self.blank_number = json_data.get('BlankNumber')
        self.amount = json_data.get('Amount')

        self.json_data = json_data


class AeroexpressOrderItemCustomerInfo(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.order_item_blank_id = json_data.get('OrderItemBlankId')
        self.order_customer_id = json_data.get('OrderCustomerId')
        self.order_item_customer_id = json_data.get('OrderItemCustomerId')
        self.amount = json_data.get('Amount')
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)

        self.json_data = json_data


class AeroexpressFullOrderItemInfo(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.order_item_customers = get_array(json_data.get('OrderItemCustomers'), AeroexpressOrderItemCustomerInfo)
        self.order_item_blanks = get_array(json_data.get('OrderItemBlanks'), AeroexpressOrderItemBlankInfo)
        self.arrival_date_time = get_datetime(json_data.get('ArrivalDateTime'))
        self.origin_location_code = json_data.get('OriginLocationCode')
        self.origin_location_name = json_data.get('OriginLocationName')
        self.destination_location_code = json_data.get('DestinationLocationCode')
        self.destination_location_name = json_data.get('DestinationLocationName')
        self.order_id = json_data.get('OrderId')
        self.agent_reference_id = json_data.get('AgentReferenceId')
        self.order_item_id = json_data.get('OrderItemId')
        self.pos_sys_name = json_data.get('PosSysName')
        self.amount = json_data.get('Amount')
        self.reservation_number = json_data.get('ReservationNumber')
        self.operation_type = json_data.get('OperationType')
        self.simple_operation_status = json_data.get('SimpleOperationStatus')
        self.detailed_operation_status = json_data.get('DetailedOperationStatus')
        self.departure_date_time = get_datetime(json_data.get('DepartureDateTime'))
        self.create_date_time = get_datetime(json_data.get('CreateDateTime'))
        self.confirm_time_limit = get_datetime(json_data.get('ConfirmTimeLimit'))
        self.confirm_date_time = get_datetime(json_data.get('ConfirmDateTime'))
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)
        self.provider_payment_form = json_data.get('ProviderPaymentForm')
        self.is_externally_loaded = json_data.get('IsExternallyLoaded')


class AeroexpressShortOrderItemInfo(object):
    def __init__(self, json_data):
        self.type = json_data.get('$type')
        self.order_id = json_data.get('OrderId')
        self.agent_reference_id = json_data.get('AgentReferenceId')
        self.order_item_id = json_data.get('OrderItemId')
        self.pos_sys_name = json_data.get('PosSysName')
        self.amount = json_data.get('Amount')
        self.reservation_number = json_data.get('ReservationNumber')
        self.operation_type = json_data.get('OperationType')
        self.simple_operation_status = json_data.get('SimpleOperationStatus')
        self.detailed_operation_status = json_data.get('DetailedOperationStatus')
        self.departure_date_time =  get_datetime(json_data.get('DepartureDateTime'))
        self.create_date_time = get_datetime(json_data.get('CreateDateTime'))
        self.confirm_time_limit =  get_datetime(json_data.get('ConfirmTimeLimit'))
        self.confirm_date_time =  get_datetime(json_data.get('ConfirmDateTime'))
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)
        self.provider_payment_form = json_data.get('ProviderPaymentForm')
        self.is_externally_loaded = json_data.get('IsExternallyLoaded')

        self.json_data = json_data


class AeroexpressShortOrderInfo(object):
    def __init__(self, json_data):
        self.order_items = get_array(json_data.get('OrderItems'), AeroexpressShortOrderItemInfo)
        self.order_id = json_data.get('OrderId')
        self.amount = json_data.get('Amount')
        self.contact_phone = json_data.get('ContactPhone')
        self.contact_emails = json_data.get('ContactEmails')
        self.created = get_datetime(json_data.get('Created'))
        self.confirmed = get_datetime(json_data.get('Confirmed'))
        self.pos_sys_name = json_data.get('PosSysName')

        self.json_data = json_data



from onelya_sdk import utils
from datetime import datetime
from onelya_sdk.wrapper.types import (RailwayPassengerCategory, PreferredAdultTariffType, CarType, CarStorey,
                                      RzhdCardTypes, CabinGenderKind, CabinPlaceDemands, ProviderPaymentForm,
                                      AdditionalPlaceRequirements, DocumentType, Sex, ReturnTarget)


class RailwayPassengerRequest(object):
    def __init__(self, category: RailwayPassengerCategory, order_customer_index: int,
                 preferred_adult_tariff_type: PreferredAdultTariffType=PreferredAdultTariffType.FULL,
                 railway_bonus_cards: 'list of RailwayBonusCardInfo'=None, is_invalid: bool=None):
        self.category = category
        self.order_customer_index = order_customer_index
        self.preferred_adult_tariff_type = preferred_adult_tariff_type
        self.railway_bonus_cards = railway_bonus_cards
        self.is_invalid = is_invalid

        self.json_data = {
            'category': self.category,
            'order_customer_index': self.order_customer_index,
            'preferred_adult_tariff_type': self.preferred_adult_tariff_type,
            'railway_bonus_cards': [item.json_data for item in self.railway_bonus_cards] if self.railway_bonus_cards is not None else [],
            'is_invalid': self.is_invalid
        }


class RailwayBonusCardInfo(object):
    def __init__(self, card_number: str, car_type: RzhdCardTypes=None):
        self.card_number = card_number
        self.car_type = car_type

        self.json_data = {
            'card_number': self.card_number,
            'car_type': self.car_type
        }


class OrderFullCustomerRequest(object):
    def __init__(self, document_number: str, document_type: DocumentType, first_name: str, last_name: str, sex: Sex,
                 index: int, middle_name: str=None, document_valid_till: datetime=None, citizenship_code: str=None,
                 birth_place: str=None, birthday: datetime=datetime.fromtimestamp(0)):
        self.type = 'ApiContracts.Order.V1.Reservation.OrderFullCustomerRequest, ApiContracts'
        self.document_number = document_number
        self.document_type = document_type
        self.first_name = first_name
        self.last_name = last_name
        self.sex = sex
        self.index = index
        self.middle_name = middle_name if middle_name is not None else '-'
        self.document_valid_till = document_valid_till
        self.citizenship_code = citizenship_code
        self.birth_place = birth_place
        self.birthday = birthday

        self.json_data = {
            'type': self.type,
            'document_number': self.document_number,
            'document_type': self.document_type,
            'first_name': self.first_name,
            'last_name': self.last_name,
            'sex': self.sex,
            'index': self.index,
            'middle_name': self.middle_name,
            'document_valid_till': utils.str_datetime(self.document_valid_till),
            'citizenship_code': self.citizenship_code,
            'birth_place': self.birth_place,
            'birthday': utils.str_datetime(self.birthday)
        }


class RailwayReservationRequest(object):
    def __init__(self, origin_code: str, destination_code: str, departure_date: datetime, train_number: str,
                 car_type: CarType, passengers: 'list of RailwayPassengerRequest', index: int, car_number: str=None,
                 lower_place_quantity: int=None, upper_place_quantity: int=None, cabin_gender_kind: CabinGenderKind=None,
                 car_storey: CarStorey=None, place_range: "tuple (min: int, max: int))"=None, bedding: bool=None,
                 cabin_place_demands: CabinPlaceDemands=None, set_electronic_registration: bool=None,
                 service_class: str=None, additional_place_requirements: AdditionalPlaceRequirements=None,
                 international_service_class: str=None, provider_payment_form: ProviderPaymentForm=None,
                 agent_reference_id: str=None, agent_payment_id: str=None):
        self.type = 'ApiContracts.Railway.V1.Messages.Reservation.RailwayReservationRequest, ApiContracts'
        self.origin_code = origin_code
        self.destination_code = destination_code
        self.departure_date = departure_date
        self.train_number = train_number
        self.car_type = car_type
        self.passengers = passengers
        self.index = index
        self.car_number = car_number
        self.lower_place_quantity = lower_place_quantity
        self.upper_place_quantity = upper_place_quantity
        self.cabin_gender_kind = cabin_gender_kind
        self.car_storey = car_storey
        self.place_range = None if place_range is None else {'From': place_range[0], 'To': place_range[1]}
        self.cabin_place_demands = cabin_place_demands
        self.set_electronic_registration = set_electronic_registration
        self.bedding = bedding
        self.service_class = service_class
        self.international_service_class = international_service_class
        self.additional_place_requirements = additional_place_requirements
        self.provider_payment_form = provider_payment_form
        self.agent_reference_id = agent_reference_id
        self.agent_payment_id = agent_payment_id

        self.json_data = {
            'type': self.type,
            'origin_code': self.origin_code,
            'destination_code': self.destination_code,
            'departure_date': utils.str_datetime(self.departure_date),
            'train_number': self.train_number,
            'car_type': self.car_type,
            'passengers': [item.json_data for item in self.passengers],
            'index': self.index,
            'car_number': self.car_number,
            'lower_place_quantity': self.lower_place_quantity,
            'upper_place_quantity': self.upper_place_quantity,
            'cabin_gender_kind': self.cabin_gender_kind,
            'car_storey': self.car_storey,
            'place_range': self.place_range,
            'cabin_place_demands': self.cabin_place_demands,
            'set_electronic_registration': self.set_electronic_registration,
            'bedding': self.bedding,
            'service_class': self.service_class,
            'international_service_class': self.international_service_class,
            'additional_place_requirements': self.additional_place_requirements,
            'provider_payment_form': self.provider_payment_form,
            'agent_reference_id': self.agent_reference_id,
            'agent_payment_id': self.agent_payment_id
        }


class OrderCustomerDocuments(object):
    def __init__(self, order_customer_id: int=None, document_number: str=None, document_type: DocumentType=None,
                 document_valid_till: datetime=None, citizenship: str=None):
        self.order_customer_id = order_customer_id
        self.document_number = document_number
        self.document_type = document_type
        self.document_valid_till = document_valid_till
        self.citizenship = citizenship


class ServiceReturnAmountRequest(object):
    def __init__(self, check_document_number: str, order_item_id: int, order_item_blank_ids: 'list of int'=None,
                 return_target: ReturnTarget=None):
        self.type = 'ApiContracts.Railway.V1.Messages.Return.RailwayReturnAmountRequest, ApiContracts'
        self.check_document_number = check_document_number
        self.order_item_id = order_item_id
        self.order_item_blank_ids = order_item_blank_ids
        self.return_target = return_target


class ServiceAutoReturnRequest(object):
    def __init__(self, check_document_number: str, order_item_id: int, order_item_blank_ids: 'list of int'=None,
                 agent_reference_id: str=None):
        self.type = 'ApiContracts.Railway.V1.Messages.Return.RailwayAutoReturnRequest, ApiContracts'
        self.check_document_number = check_document_number
        self.order_item_id = order_item_id
        self.order_item_blank_ids = order_item_blank_ids
        self.agent_reference_id = agent_reference_id


class ProductRequest(object):
    def __init__(self, package: str):
        self.type = 'ApiContracts.Insurance.V1.Products.Travel.AddUpsale.AviaAddUpsaleRequest, ApiContracts'
        self.package = package


class ServiceAddUpsaleRequest(object):
    def __init__(self, supplier: str, order_customer_ids: 'list of int', product_request: ProductRequest,
                 agent_payment_id: int=None):
        self.type = 'ApiContracts.Insurance.V1.Messages.InsuranceAddUpsaleRequest, ApiContracts'
        self.supplier = supplier
        self.order_customer_ids = order_customer_ids
        self.product_request = product_request
        self.agent_payment_id = agent_payment_id



from .session import Session
from .utils import get_array, get_item
from .wrapper.requests import RequestWrapper
from .railway.info import Info as RailwayInfo
from .references import References as References
from .railway.search import Search as RailwaySearch
from .railway.additional_meal import AdditionalMeal
from .aeroexpress.info import Info as AeroexpressInfo
from .wrapper import AgentAccount, RailwayPricingResponse
from .aeroexpress.search import Search as AeroexpressSearch
from .railway.reservation import Reservation as RailwayReservation
from .aeroexpress.reservation import Reservation as AeroexpressReservation

__version__ = 0.1

BALANCES_METHOD = 'Partner/V1/Info/Balances'
PRICING_METHOD = 'Insurance/V1/Search/Pricing'


class API(object):
    def __init__(self, username: str, password: str, pos: str, ssl_verify: bool=True):
        self.__session = Session(username, password, pos, ssl_verify)
        self.__request_wrapper = RequestWrapper(self.__session)

        self.railway_search = RailwaySearch(self.__request_wrapper)
        self.railway_reservation = RailwayReservation(self.__request_wrapper)
        self.railway_info = RailwayInfo(self.__request_wrapper)
        self.additional_meal = AdditionalMeal(self.__request_wrapper)

        self.aeroexpress_search = AeroexpressSearch(self.__request_wrapper)
        self.aeroexpress_reservation = AeroexpressReservation(self.__request_wrapper)
        self.aeroexpress_info = AeroexpressInfo(self.__request_wrapper)

        self.references = References(self.__request_wrapper)

    def partner_balances(self):
        response = self.__request_wrapper.make_request(BALANCES_METHOD)
        return Balances(response)

    def railway_search_pricing(self):
        response = self.__request_wrapper.make_request(PRICING_METHOD, json={
            'Product': {
                '$type': 'ApiContracts.Insurance.V1.Products.Travel.Pricing.RailwayPricingRequest, ApiContracts'
            }
        })
        return Pricing(response)

    @property
    def last_response(self):
        return self.__session.last_response_data
    
    @property
    def last_request(self):
        return self.__session.last_request_data


class Balances(object):
    def __init__(self, json_data):
        self.account_balances = get_array(json_data.get('AccountBalances', None), AgentAccount)

        self.json_data = json_data


class Pricing(object):
    def __init__(self, json_data):
        self.pricing_response = get_item(json_data.get('PricingResponse', None), RailwayPricingResponse)

        self.json_data = json_data



from datetime import datetime
from ..utils import str_datetime


class RequestWrapper(object):
    def __init__(self, session):
        self.session = session
        self.method_name = None

    def make_request(self, method_name, json=None, **kwargs):
        self.method_name = method_name
        json_data = self.__get_json_data(False, **kwargs) if json is None else json
        return self.session.make_api_request(method_name, json_data)

    def __get_json_data(self, recursive, **kwargs):
        json_data = {}
        for key in kwargs.keys():
            item = kwargs[key]
            onelya_key = self.__get_onelya_key(key)
            if type(item) in [str, int, float, type(None), bool]:
                json_data[onelya_key] = item
            elif type(item) is dict:
                json_data[onelya_key] = self.__get_json_data(False, **item)
            elif type(item) is list:
                json_data[onelya_key] = [self.__get_json_data(True, **{'Key': list_item}) for list_item in item]
            elif type(item) is datetime:
                json_data[onelya_key] = str_datetime(item)
            else:
                object_name = self.__get_onelya_key(key)
                if json_data.get(object_name, None) is None:
                    json_data[object_name] = {}
                for object_attribute in item.__dict__.keys():
                    if object_attribute != 'json_data':
                        object_attribute_name = self.__get_onelya_key(object_attribute)
                        json_data[object_name][object_attribute_name] = \
                            self.__get_json_data(True, **{object_attribute_name: item.__getattribute__(object_attribute)})
            if recursive:
                return json_data[key]
        return json_data

    def __get_onelya_key(self, key):
        """Getting json key
        Also converting type -> $type if method not from references

        :param key:
        :param method_name:
        :return:
        """
        if key == 'type' and 'References' not in self.method_name:
            return '$type'
        new_key = key[0].upper()
        i = 1
        while i < len(key):
            if key[i] == '_':
                new_key += key[i + 1].upper()
                i += 1
            else:
                new_key += key[i]
            i += 1
        return new_key





from datetime import datetime
from onelya_sdk import utils
from onelya_sdk.wrapper.types import (RailwayPassengerCategory, PreferredAdultTariffType, CarType, CarStorey,
                                      RzhdCardTypes, CabinGenderKind, CabinPlaceDemands, ProviderPaymentForm,
                                      AdditionalPlaceRequirements, DocumentType, Sex)


class AeroexpressPassengerRequest(object):
    def __init__(self, order_customer_index):
        self.order_customer_index = order_customer_index


class OrderFullCustomerRequest(object):
    def __init__(self, document_number: str, document_type: DocumentType, first_name: str, last_name: str, sex: Sex,
                 index: int, middle_name: str=None, document_valid_till: datetime=None, citizenship_code: str=None,
                 birth_place: str=None, birthday: datetime=datetime.fromtimestamp(0)):
        self.type = 'ApiContracts.Order.V1.Reservation.OrderFullCustomerRequest, ApiContracts'
        self.document_number = document_number
        self.document_type = document_type
        self.first_name = first_name
        self.last_name = last_name
        self.sex = sex
        self.index = index
        self.middle_name = middle_name if middle_name is not None else '-'
        self.document_valid_till = document_valid_till
        self.citizenship_code = citizenship_code
        self.birth_place = birth_place
        self.birthday = birthday

        self.json_data = {
            'type': self.type,
            'document_number': self.document_number,
            'document_type': self.document_type,
            'first_name': self.first_name,
            'last_name': self.last_name,
            'sex': self.sex,
            'index': self.index,
            'middle_name': self.middle_name,
            'document_valid_till': utils.str_datetime(self.document_valid_till),
            'citizenship_code': self.citizenship_code,
            'birth_place': self.birth_place,
            'birthday': utils.str_datetime(self.birthday)
        }


class AeroexpressReservationRequest(object):
    def __init__(self, tariff_id: str, departure_date: datetime, passengers: 'list of int', index: int,
                 schedule_id: str=None, provider_payment_form: ProviderPaymentForm=None, agent_reference_id: str=None,
                 agent_payment_id: str=None):
        self.type = 'ApiContracts.Aeroexpress.V1.Messages.Reservation.AeroexpressReservationRequest, ApiContracts'
        self.tariff_id = tariff_id
        self.departure_date = departure_date
        self.passengers = utils.get_array(passengers, AeroexpressPassengerRequest)
        self.index = index
        self.schedule_id = schedule_id
        self.provider_payment_form = provider_payment_form
        self.agent_reference_id = agent_reference_id
        self.agent_payment_id = agent_payment_id

        self.json_data = {
            'type': self.type,
            'tariff_id': self.tariff_id,
            'departure_date': utils.str_datetime(self.departure_date),
            'passengers': [item.order_customer_index for item in self.passengers],
            'index': self.index,
            'schedule_id': self.schedule_id,
            'provider_payment_form': self.provider_payment_form,
            'agent_reference_id': self.agent_reference_id,
            'agent_payment_id': self.agent_payment_id
        }


class OrderCustomerDocuments(object):
    def __init__(self, order_customer_id: int=None, document_number: str=None, document_type: DocumentType=None,
                 document_valid_till: datetime=None, citizenship: str=None):
        self.order_customer_id = order_customer_id
        self.document_number = document_number
        self.document_type = document_type
        self.document_valid_till = document_valid_till
        self.citizenship = citizenship


class AeroexpressAutoReturnRequest(object):
    def __init__(self, order_item_id: int, agent_reference_id: str=None):
        self.type = 'ApiContracts.Aeroexpress.V1.Messages.Return.AeroexpressAutoReturnRequest, ApiContracts'
        self.order_item_id = order_item_id
        self.agent_reference_id = agent_reference_id


class ProductRequest(object):
    def __init__(self, package: str):
        self.type = 'ApiContracts.Insurance.V1.Products.Travel.AddUpsale.AviaAddUpsaleRequest, ApiContracts'
        self.package = package



from datetime import datetime
from onelya_sdk.utils import get_array, get_item, get_money
from onelya_sdk.wrapper import FeeCalculation, RaceInfo, TariffPriceInfoResponse

TARIFF_PRICING_METHOD = 'Aeroexpress/V1/Search/TariffPricing'
TARIFF_PRICE_INFO_METHOD = 'Aeroexpress/V1/Search/TariffPriceInfo'


class Search(object):
    def __init__(self, request_wrapper):
        self.__request_wrapper = request_wrapper

    def tariff_pricing(self, departure_date: datetime):
        response = self.__request_wrapper.make_request(TARIFF_PRICING_METHOD, departure_date=departure_date)
        return TariffPricing(response)

    def tariff_price_info(self, departure_date: datetime, tariff_id: str):
        response = self.__request_wrapper.make_request(TARIFF_PRICE_INFO_METHOD, departure_date=departure_date,
                                                       tariff_id=tariff_id)
        return TariffPriceInfo(response)


class TariffPricing(object):
    def __init__(self, json_data):
        self.tariffs = get_array(json_data.get('Tariffs'), TariffPriceInfoResponse)

        self.json_data = json_data


class TariffPriceInfo(object):
    def __init__(self, json_data):
        self.document_types = json_data.get('DocumentTypes')
        self.client_fee_calculation = get_item(json_data.get('ClientFeeCalculation'), FeeCalculation)
        self.agent_fee_calculation = get_item(json_data.get('AgentFeeCalculation'), FeeCalculation)
        self.tariff_id = json_data.get('TariffId')
        self.tariff_name = json_data.get('TariffName')
        self.tariff_type = json_data.get('TariffType')
        self.route_name = json_data.get('RouteName')
        self.description = json_data.get('Description')
        self.price = get_money(json_data.get('Price'))
        self.max_tickets_quantity_allowed_for_booking = json_data.get('MaxTicketsQuantityAllowedForBooking')
        self.is_for_guaranteed_seats = json_data.get('IsForGuaranteedSeats')
        self.races = get_array(json_data.get('Races'), RaceInfo)

        self.json_data = json_data



from setuptools import setup, find_packages

__version__ = '1.4.4'

setup(
    version=__version__,
    name='onelya_sdk',
    packages=find_packages(),

    install_requires=[
        'requests'
    ],

    description='Onelya Python SDK',

    author='Travel Managment Consulting',
    author_email='otd@tm-consulting.ru',

    url='https://github.com/tmconsulting/onelya-python-sdk',
    download_url='https://github.com/tmconsulting/onelya-python-sdk/archive/%s.tar.gz' % __version__,

    license='MIT License',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



import json
import requests
from requests.auth import HTTPBasicAuth
from .exceptions import OnelyaAPIError


class Session(object):
    API_URL = 'https://api-test.onelya.ru/'

    def __init__(self, username, password, pos, ssl_verify):
        self.pos = pos

        self.username = username
        self.password = password

        self.requests_session = requests.Session()
        self.requests_session.verify = ssl_verify

        self.requests_session.headers['Pos'] = self.pos
        self.requests_session.headers['Content-Encoding'] = 'gzip'
        self.requests_session.headers['Content-Type'] = 'application/json; charset=utf-8'
        self.requests_session.auth = HTTPBasicAuth(self.username, self.password)

        self.last_response_data = None
        self.last_request_data = None

    def make_api_request(self, method, data):
        response = self.__send_api_request(method, data)
        try:
            response = response.json()
        except ValueError:
            if response.headers['Content-Type'] == 'application/pdf':
                self.last_response_data = response.content
            else:
                self.last_response_data = response.text
            return response

        self.last_response_data = response

        if 'Code' in response:
            raise OnelyaAPIError(method, response, data)
        return response

    def __send_api_request(self, method, data):
        url = '{}{}'.format(Session.API_URL, method)
        self.last_request_data = data
        response = self.requests_session.post(url, data=json.dumps(data), timeout=120)
        return response
